package javax.swing.plaf.basic;

import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.event.FocusAdapter;
import java.awt.event.MouseAdapter;
import java.awt.event.ContainerEvent;
import java.awt.event.FocusEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.ContainerListener;
import javax.swing.JPanel;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import javax.swing.JViewport;
import java.awt.Container;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.awt.Point;
import javax.swing.plaf.UIResource;
import java.awt.Polygon;
import javax.swing.Icon;
import java.awt.Font;
import sun.swing.SwingUtilities2;
import java.awt.Graphics;
import java.awt.FontMetrics;
import java.awt.Dimension;
import javax.swing.plaf.ComponentInputMapUIResource;
import sun.swing.DefaultLookup;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.LookAndFeel;
import javax.swing.JButton;
import java.awt.LayoutManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.InputMap;
import java.util.Hashtable;
import javax.swing.text.View;
import java.util.Vector;
import java.awt.Component;
import java.awt.event.FocusListener;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeListener;
import java.awt.Rectangle;
import javax.swing.KeyStroke;
import java.awt.Insets;
import java.awt.Color;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.plaf.TabbedPaneUI;

public class BasicTabbedPaneUI extends TabbedPaneUI implements SwingConstants
{
    protected JTabbedPane tabPane;
    protected Color highlight;
    protected Color lightHighlight;
    protected Color shadow;
    protected Color darkShadow;
    protected Color focus;
    private Color selectedColor;
    protected int textIconGap;
    protected int tabRunOverlay;
    protected Insets tabInsets;
    protected Insets selectedTabPadInsets;
    protected Insets tabAreaInsets;
    protected Insets contentBorderInsets;
    private boolean tabsOverlapBorder;
    private boolean tabsOpaque;
    private boolean contentOpaque;
    @Deprecated
    protected KeyStroke upKey;
    @Deprecated
    protected KeyStroke downKey;
    @Deprecated
    protected KeyStroke leftKey;
    @Deprecated
    protected KeyStroke rightKey;
    protected int[] tabRuns;
    protected int runCount;
    protected int selectedRun;
    protected Rectangle[] rects;
    protected int maxTabHeight;
    protected int maxTabWidth;
    protected ChangeListener tabChangeListener;
    protected PropertyChangeListener propertyChangeListener;
    protected MouseListener mouseListener;
    protected FocusListener focusListener;
    private Insets currentPadInsets;
    private Insets currentTabAreaInsets;
    private Component visibleComponent;
    private Vector<View> htmlViews;
    private Hashtable<Integer, Integer> mnemonicToIndexMap;
    private InputMap mnemonicInputMap;
    private BasicTabbedPaneUI.ScrollableTabSupport tabScroller;
    private BasicTabbedPaneUI.TabContainer tabContainer;
    protected transient Rectangle calcRect;
    private int focusIndex;
    private BasicTabbedPaneUI.Handler handler;
    private int rolloverTabIndex;
    private boolean isRunsDirty;
    private boolean calculatedBaseline;
    private int baseline;
    private static int[] xCropLen;
    private static int[] yCropLen;
    private static final int CROP_SEGMENT = 12;
    
    public BasicTabbedPaneUI() {
        this.tabsOpaque = true;
        this.contentOpaque = true;
        this.tabRuns = new int[10];
        this.runCount = 0;
        this.selectedRun = -1;
        this.rects = new Rectangle[0];
        this.currentPadInsets = new Insets(0, 0, 0, 0);
        this.currentTabAreaInsets = new Insets(0, 0, 0, 0);
        this.calcRect = new Rectangle(0, 0, 0, 0);
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicTabbedPaneUI();
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigateNext"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigatePrevious"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigateRight"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigateLeft"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigateUp"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigateDown"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigatePageUp"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("navigatePageDown"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("requestFocus"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("requestFocusForVisibleComponent"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("setSelectedIndex"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("selectTabWithFocus"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("scrollTabsForwardAction"));
        lazyActionMap.put(new BasicTabbedPaneUI.Actions("scrollTabsBackwardAction"));
    }
    
    @Override
    public void installUI(JComponent component) {
        this.tabPane = (JTabbedPane)component;
        this.calculatedBaseline = false;
        this.rolloverTabIndex = -1;
        this.focusIndex = -1;
        component.setLayout(this.createLayoutManager());
        this.installComponents();
        this.installDefaults();
        this.installListeners();
        this.installKeyboardActions();
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallKeyboardActions();
        this.uninstallListeners();
        this.uninstallDefaults();
        this.uninstallComponents();
        component.setLayout(null);
        this.tabPane = null;
    }
    
    protected LayoutManager createLayoutManager() {
        if (this.tabPane.getTabLayoutPolicy() == 1) {
            return new BasicTabbedPaneUI.TabbedPaneScrollLayout();
        }
        return new BasicTabbedPaneUI.TabbedPaneLayout();
    }
    
    private boolean scrollableTabLayoutEnabled() {
        return this.tabPane.getLayout() instanceof BasicTabbedPaneUI.TabbedPaneScrollLayout;
    }
    
    protected void installComponents() {
        if (this.scrollableTabLayoutEnabled() && this.tabScroller == null) {
            this.tabScroller = new BasicTabbedPaneUI.ScrollableTabSupport(this.tabPane.getTabPlacement());
            this.tabPane.add(this.tabScroller.viewport);
        }
        this.installTabContainer();
    }
    
    private void installTabContainer() {
        for (int i = 0; i < this.tabPane.getTabCount(); ++i) {
            Component tabComponent = this.tabPane.getTabComponentAt(i);
            if (tabComponent != null) {
                if (this.tabContainer == null) {
                    this.tabContainer = new BasicTabbedPaneUI.TabContainer();
                }
                this.tabContainer.add(tabComponent);
            }
        }
        if (this.tabContainer == null) {
            return;
        }
        if (this.scrollableTabLayoutEnabled()) {
            this.tabScroller.tabPanel.add(this.tabContainer);
        }
        else {
            this.tabPane.add(this.tabContainer);
        }
        return;
    }
    
    protected JButton createScrollButton(int n) {
        if (n != 5 && n != 1 && n != 3 && n != 7) {
            throw new IllegalArgumentException("Direction must be one of: SOUTH, NORTH, EAST or WEST");
        }
        return new BasicTabbedPaneUI.ScrollableTabButton(n);
    }
    
    protected void uninstallComponents() {
        this.uninstallTabContainer();
        if (this.scrollableTabLayoutEnabled()) {
            this.tabPane.remove(this.tabScroller.viewport);
            this.tabPane.remove(this.tabScroller.scrollForwardButton);
            this.tabPane.remove(this.tabScroller.scrollBackwardButton);
            this.tabScroller = null;
        }
    }
    
    private void uninstallTabContainer() {
        if (this.tabContainer == null) {
            return;
        }
        this.tabContainer.notifyTabbedPane = false;
        this.tabContainer.removeAll();
        if (this.scrollableTabLayoutEnabled()) {
            this.tabContainer.remove(this.tabScroller.croppedEdge);
            this.tabScroller.tabPanel.remove(this.tabContainer);
        }
        else {
            this.tabPane.remove(this.tabContainer);
        }
        this.tabContainer = null;
        return;
    }
    
    protected void installDefaults() {
        LookAndFeel.installColorsAndFont(this.tabPane, "TabbedPane.background", "TabbedPane.foreground", "TabbedPane.font");
        this.highlight = UIManager.getColor("TabbedPane.light");
        this.lightHighlight = UIManager.getColor("TabbedPane.highlight");
        this.shadow = UIManager.getColor("TabbedPane.shadow");
        this.darkShadow = UIManager.getColor("TabbedPane.darkShadow");
        this.focus = UIManager.getColor("TabbedPane.focus");
        this.selectedColor = UIManager.getColor("TabbedPane.selected");
        this.textIconGap = UIManager.getInt("TabbedPane.textIconGap");
        this.tabInsets = UIManager.getInsets("TabbedPane.tabInsets");
        this.selectedTabPadInsets = UIManager.getInsets("TabbedPane.selectedTabPadInsets");
        this.tabAreaInsets = UIManager.getInsets("TabbedPane.tabAreaInsets");
        this.tabsOverlapBorder = UIManager.getBoolean("TabbedPane.tabsOverlapBorder");
        this.contentBorderInsets = UIManager.getInsets("TabbedPane.contentBorderInsets");
        this.tabRunOverlay = UIManager.getInt("TabbedPane.tabRunOverlay");
        this.tabsOpaque = UIManager.getBoolean("TabbedPane.tabsOpaque");
        this.contentOpaque = UIManager.getBoolean("TabbedPane.contentOpaque");
        Object propertyValue = UIManager.get("TabbedPane.opaque");
        if (propertyValue == null) {
            propertyValue = Boolean.FALSE;
        }
        LookAndFeel.installProperty(this.tabPane, "opaque", propertyValue);
        if (this.tabInsets == null) {
            this.tabInsets = new Insets(0, 4, 1, 4);
        }
        if (this.selectedTabPadInsets == null) {
            this.selectedTabPadInsets = new Insets(2, 2, 2, 1);
        }
        if (this.tabAreaInsets == null) {
            this.tabAreaInsets = new Insets(3, 2, 0, 2);
        }
        if (this.contentBorderInsets == null) {
            this.contentBorderInsets = new Insets(2, 2, 3, 3);
        }
    }
    
    protected void uninstallDefaults() {
        this.highlight = null;
        this.lightHighlight = null;
        this.shadow = null;
        this.darkShadow = null;
        this.focus = null;
        this.tabInsets = null;
        this.selectedTabPadInsets = null;
        this.tabAreaInsets = null;
        this.contentBorderInsets = null;
    }
    
    protected void installListeners() {
        PropertyChangeListener propertyChangeListener = this.createPropertyChangeListener();
        this.propertyChangeListener = propertyChangeListener;
        if (propertyChangeListener != null) {
            this.tabPane.addPropertyChangeListener(this.propertyChangeListener);
        }
        if ((this.tabChangeListener = this.createChangeListener()) != null) {
            this.tabPane.addChangeListener(this.tabChangeListener);
        }
        if ((this.mouseListener = this.createMouseListener()) != null) {
            this.tabPane.addMouseListener(this.mouseListener);
        }
        this.tabPane.addMouseMotionListener(this.getHandler());
        if ((this.focusListener = this.createFocusListener()) != null) {
            this.tabPane.addFocusListener(this.focusListener);
        }
        this.tabPane.addContainerListener(this.getHandler());
        if (this.tabPane.getTabCount() > 0) {
            this.htmlViews = this.createHTMLVector();
        }
    }
    
    protected void uninstallListeners() {
        if (this.mouseListener != null) {
            this.tabPane.removeMouseListener(this.mouseListener);
            this.mouseListener = null;
        }
        this.tabPane.removeMouseMotionListener(this.getHandler());
        if (this.focusListener != null) {
            this.tabPane.removeFocusListener(this.focusListener);
            this.focusListener = null;
        }
        this.tabPane.removeContainerListener(this.getHandler());
        if (this.htmlViews != null) {
            this.htmlViews.removeAllElements();
            this.htmlViews = null;
        }
        if (this.tabChangeListener != null) {
            this.tabPane.removeChangeListener(this.tabChangeListener);
            this.tabChangeListener = null;
        }
        if (this.propertyChangeListener != null) {
            this.tabPane.removePropertyChangeListener(this.propertyChangeListener);
            this.propertyChangeListener = null;
        }
        this.handler = null;
    }
    
    protected MouseListener createMouseListener() {
        return this.getHandler();
    }
    
    protected FocusListener createFocusListener() {
        return this.getHandler();
    }
    
    protected ChangeListener createChangeListener() {
        return this.getHandler();
    }
    
    protected PropertyChangeListener createPropertyChangeListener() {
        return this.getHandler();
    }
    
    private BasicTabbedPaneUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicTabbedPaneUI.Handler();
        }
        return this.handler;
    }
    
    protected void installKeyboardActions() {
        SwingUtilities.replaceUIInputMap(this.tabPane, 1, this.getInputMap(1));
        SwingUtilities.replaceUIInputMap(this.tabPane, 0, this.getInputMap(0));
        LazyActionMap.installLazyActionMap(this.tabPane, BasicTabbedPaneUI.class, "TabbedPane.actionMap");
        this.updateMnemonics();
    }
    
    InputMap getInputMap(int n) {
        if (n == 1) {
            return (InputMap)DefaultLookup.get(this.tabPane, this, "TabbedPane.ancestorInputMap");
        }
        if (n == 0) {
            return (InputMap)DefaultLookup.get(this.tabPane, this, "TabbedPane.focusInputMap");
        }
        return null;
    }
    
    protected void uninstallKeyboardActions() {
        SwingUtilities.replaceUIActionMap(this.tabPane, null);
        SwingUtilities.replaceUIInputMap(this.tabPane, 1, null);
        SwingUtilities.replaceUIInputMap(this.tabPane, 0, null);
        SwingUtilities.replaceUIInputMap(this.tabPane, 2, null);
        this.mnemonicToIndexMap = null;
        this.mnemonicInputMap = null;
    }
    
    private void updateMnemonics() {
        this.resetMnemonics();
        for (int i = this.tabPane.getTabCount() - 1; i >= 0; --i) {
            int mnemonic = this.tabPane.getMnemonicAt(i);
            if (mnemonic > 0) {
                this.addMnemonic(i, mnemonic);
            }
        }
    }
    
    private void resetMnemonics() {
        if (this.mnemonicToIndexMap != null) {
            this.mnemonicToIndexMap.clear();
            this.mnemonicInputMap.clear();
        }
    }
    
    private void addMnemonic(int i, int n) {
        if (this.mnemonicToIndexMap == null) {
            this.initMnemonics();
        }
        this.mnemonicInputMap.put(KeyStroke.getKeyStroke(n, BasicLookAndFeel.getFocusAcceleratorKeyMask()), "setSelectedIndex");
        this.mnemonicToIndexMap.put(Integer.valueOf(n), Integer.valueOf(i));
    }
    
    private void initMnemonics() {
        this.mnemonicToIndexMap = new Hashtable();
        (this.mnemonicInputMap = new ComponentInputMapUIResource(this.tabPane)).setParent(SwingUtilities.getUIInputMap(this.tabPane, 2));
        SwingUtilities.replaceUIInputMap(this.tabPane, 2, this.mnemonicInputMap);
    }
    
    private void setRolloverTab(int n, int n2) {
        this.setRolloverTab(this.tabForCoordinate(this.tabPane, n, n2, false));
    }
    
    protected void setRolloverTab(int rolloverTabIndex) {
        this.rolloverTabIndex = rolloverTabIndex;
    }
    
    protected int getRolloverTab() {
        return this.rolloverTabIndex;
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        return null;
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        return null;
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        int calculateBaselineIfNecessary = this.calculateBaselineIfNecessary();
        if (calculateBaselineIfNecessary != -1) {
            int tabPlacement = this.tabPane.getTabPlacement();
            Insets insets = this.tabPane.getInsets();
            Insets tabAreaInsets = this.getTabAreaInsets(tabPlacement);
            switch (tabPlacement) {
                case 1: {
                    return calculateBaselineIfNecessary + (insets.top + tabAreaInsets.top);
                }
                case 3: {
                    return height - insets.bottom - tabAreaInsets.bottom - this.maxTabHeight + calculateBaselineIfNecessary;
                }
                case 2:
                case 4: {
                    return calculateBaselineIfNecessary + (insets.top + tabAreaInsets.top);
                }
                default: {
                    break;
                }
            }
        }
        return -1;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        switch (this.tabPane.getTabPlacement()) {
            case 1:
            case 2:
            case 4: {
                return Component.BaselineResizeBehavior.CONSTANT_ASCENT;
            }
            case 3: {
                return Component.BaselineResizeBehavior.CONSTANT_DESCENT;
            }
            default: {
                return Component.BaselineResizeBehavior.OTHER;
            }
        }
    }
    
    protected int getBaseline(int n) {
        if (this.tabPane.getTabComponentAt(n) != null) {
            if (this.getBaselineOffset() != 0) {
                return -1;
            }
            Component tabComponent = this.tabPane.getTabComponentAt(n);
            Dimension preferredSize = tabComponent.getPreferredSize();
            Insets tabInsets = this.getTabInsets(this.tabPane.getTabPlacement(), n);
            return tabComponent.getBaseline(preferredSize.width, preferredSize.height) + (this.maxTabHeight - tabInsets.top - tabInsets.bottom - preferredSize.height) / 2 + tabInsets.top;
        }
        else {
            View textViewForTab = this.getTextViewForTab(n);
            if (textViewForTab != null) {
                int n2 = (int)textViewForTab.getPreferredSpan(1);
                int htmlBaseline = BasicHTML.getHTMLBaseline(textViewForTab, (int)textViewForTab.getPreferredSpan(0), n2);
                if (htmlBaseline >= 0) {
                    return this.maxTabHeight / 2 - n2 / 2 + htmlBaseline + this.getBaselineOffset();
                }
                return -1;
            }
            else {
                FontMetrics fontMetrics = this.getFontMetrics();
                return this.maxTabHeight / 2 - fontMetrics.getHeight() / 2 + fontMetrics.getAscent() + this.getBaselineOffset();
            }
        }
    }
    
    protected int getBaselineOffset() {
        switch (this.tabPane.getTabPlacement()) {
            case 1: {
                if (this.tabPane.getTabCount() > 1) {
                    return 1;
                }
                return -1;
            }
            case 3: {
                if (this.tabPane.getTabCount() > 1) {
                    return -1;
                }
                return 1;
            }
            default: {
                return this.maxTabHeight % 2;
            }
        }
    }
    
    private int calculateBaselineIfNecessary() {
        if (!this.calculatedBaseline) {
            this.calculatedBaseline = true;
            this.baseline = -1;
            if (this.tabPane.getTabCount() > 0) {
                this.calculateBaseline();
            }
        }
        return this.baseline;
    }
    
    private void calculateBaseline() {
        int tabCount = this.tabPane.getTabCount();
        int tabPlacement = this.tabPane.getTabPlacement();
        this.maxTabHeight = this.calculateMaxTabHeight(tabPlacement);
        this.baseline = this.getBaseline(0);
        if (this.isHorizontalTabPlacement()) {
            int j = 1;
            while (j < tabCount) {
                if (this.getBaseline(j) != this.baseline) {
                    this.baseline = -1;
                    break;
                }
                ++j;
            }
        }
        else {
            int height = this.getFontMetrics().getHeight();
            int calculateTabHeight = this.calculateTabHeight(tabPlacement, 0, height);
            int i = 1;
            while (i < tabCount) {
                if (calculateTabHeight != this.calculateTabHeight(tabPlacement, i, height)) {
                    this.baseline = -1;
                    break;
                }
                ++i;
            }
        }
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        int selectedIndex = this.tabPane.getSelectedIndex();
        int tabPlacement = this.tabPane.getTabPlacement();
        this.ensureCurrentLayout();
        if (this.tabsOverlapBorder) {
            this.paintContentBorder(graphics, tabPlacement, selectedIndex);
        }
        if (!this.scrollableTabLayoutEnabled()) {
            this.paintTabArea(graphics, tabPlacement, selectedIndex);
        }
        if (!this.tabsOverlapBorder) {
            this.paintContentBorder(graphics, tabPlacement, selectedIndex);
        }
    }
    
    protected void paintTabArea(Graphics graphics, int n, int n2) {
        int tabCount = this.tabPane.getTabCount();
        Rectangle rectangle = new Rectangle();
        Rectangle rectangle2 = new Rectangle();
        Rectangle clipBounds = graphics.getClipBounds();
        for (int i = this.runCount - 1; i >= 0; --i) {
            int j = this.tabRuns[i];
            int n3 = this.tabRuns[(i == this.runCount - 1) ? 0 : (i + 1)];
            int n4;
            for (n4 = ((n3 != 0) ? (n3 - 1) : (tabCount - 1)), ; j <= n4; ++j) {
                if (j != n2 && this.rects[j].intersects(clipBounds)) {
                    this.paintTab(graphics, n, this.rects, j, rectangle, rectangle2);
                }
            }
        }
        if (n2 >= 0 && this.rects[n2].intersects(clipBounds)) {
            this.paintTab(graphics, n, this.rects, n2, rectangle, rectangle2);
        }
    }
    
    protected void paintTab(Graphics g, int n, Rectangle[] array, int n2, Rectangle rectangle, Rectangle rectangle2) {
        Rectangle rectangle3 = array[n2];
        int n3 = this.tabPane.getSelectedIndex() == n2;
        if (this.tabsOpaque || this.tabPane.isOpaque()) {
            this.paintTabBackground(g, n, n2, rectangle3.x, rectangle3.y, rectangle3.width, rectangle3.height, (boolean)n3);
        }
        this.paintTabBorder(g, n, n2, rectangle3.x, rectangle3.y, rectangle3.width, rectangle3.height, (boolean)n3);
        String s = this.tabPane.getTitleAt(n2);
        Font font = this.tabPane.getFont();
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(this.tabPane, g, font);
        Icon iconForTab = this.getIconForTab(n2);
        this.layoutLabel(n, fontMetrics, n2, s, iconForTab, rectangle3, rectangle, rectangle2, (boolean)n3);
        if (this.tabPane.getTabComponentAt(n2) == null) {
            if (this.scrollableTabLayoutEnabled() && this.tabScroller.croppedEdge.isParamsSet() && this.tabScroller.croppedEdge.getTabIndex() == n2 && this.isHorizontalTabPlacement()) {
                s = SwingUtilities2.clipStringIfNecessary(null, fontMetrics, s, this.tabScroller.croppedEdge.getCropline() - (rectangle2.x - rectangle3.x) - this.tabScroller.croppedEdge.getCroppedSideWidth());
            }
            else if (!this.scrollableTabLayoutEnabled() && this.isHorizontalTabPlacement()) {
                s = SwingUtilities2.clipStringIfNecessary(null, fontMetrics, s, rectangle2.width);
            }
            this.paintText(g, n, font, fontMetrics, n2, s, rectangle2, (boolean)n3);
            this.paintIcon(g, n, n2, iconForTab, rectangle, (boolean)n3);
        }
        this.paintFocusIndicator(g, n, array, n2, rectangle, rectangle2, (boolean)n3);
    }
    
    private boolean isHorizontalTabPlacement() {
        return this.tabPane.getTabPlacement() == 1 || this.tabPane.getTabPlacement() == 3;
    }
    
    private static Polygon createCroppedTabShape(int n, Rectangle rectangle, int n2) {
        int n3;
        int n4;
        int n5;
        int n6;
        switch (n) {
            case 2:
            case 4: {
                n3 = rectangle.width;
                n4 = rectangle.x;
                n5 = rectangle.x + rectangle.width;
                n6 = rectangle.y + rectangle.height;
                break;
            }
            default: {
                n3 = rectangle.height;
                n4 = rectangle.y;
                n5 = rectangle.y + rectangle.height;
                n6 = rectangle.x + rectangle.width;
                break;
            }
        }
        int n7 = n3 / 12;
        if (n3 % 12 > 0) {
            ++n7;
        }
        int n8 = 2 + n7 * 8;
        int[] array = new int[n8];
        int[] array2 = new int[n8];
        int n9 = 0;
        array[n9] = n6;
        array2[n9++] = n5;
        array[n9] = n6;
        array2[n9++] = n4;
        for (int i = 0; i < n7; ++i) {
            int j = 0;
            while (j < BasicTabbedPaneUI.xCropLen.length) {
                array[n9] = n2 - BasicTabbedPaneUI.xCropLen[j];
                array2[n9] = n4 + i * 12 + BasicTabbedPaneUI.yCropLen[j];
                if (array2[n9] >= n5) {
                    array2[n9] = n5;
                    ++n9;
                    break;
                }
                ++n9;
                ++j;
            }
        }
        if (n == 1 || n == 3) {
            return new Polygon(array, array2, n9);
        }
        return new Polygon(array2, array, n9);
    }
    
    private void paintCroppedTabEdge(Graphics graphics) {
        int tabIndex = this.tabScroller.croppedEdge.getTabIndex();
        int cropline = this.tabScroller.croppedEdge.getCropline();
        switch (this.tabPane.getTabPlacement()) {
            case 2:
            case 4: {
                int i = this.rects[tabIndex].x;
                graphics.setColor(this.shadow);
                while (i <= i + this.rects[tabIndex].width) {
                    for (int j = 0; j < BasicTabbedPaneUI.xCropLen.length; j += 2) {
                        graphics.drawLine(i + BasicTabbedPaneUI.yCropLen[j], cropline - BasicTabbedPaneUI.xCropLen[j], i + BasicTabbedPaneUI.yCropLen[j + 1] - 1, cropline - BasicTabbedPaneUI.xCropLen[j + 1]);
                    }
                    i += 12;
                }
                break;
            }
            default: {
                int k = this.rects[tabIndex].y;
                graphics.setColor(this.shadow);
                while (k <= k + this.rects[tabIndex].height) {
                    for (int l = 0; l < BasicTabbedPaneUI.xCropLen.length; l += 2) {
                        graphics.drawLine(cropline - BasicTabbedPaneUI.xCropLen[l], k + BasicTabbedPaneUI.yCropLen[l], cropline - BasicTabbedPaneUI.xCropLen[l + 1], k + BasicTabbedPaneUI.yCropLen[l + 1] - 1);
                    }
                    k += 12;
                }
                break;
            }
        }
    }
    
    protected void layoutLabel(int n, FontMetrics fm, int n2, String text, Icon icon, Rectangle viewR, Rectangle iconR, Rectangle textR, boolean b) {
        int n3 = 0;
        iconR.y = n3;
        iconR.x = n3;
        textR.y = n3;
        textR.x = n3;
        View textViewForTab = this.getTextViewForTab(n2);
        if (textViewForTab != null) {
            this.tabPane.putClientProperty("html", textViewForTab);
        }
        SwingUtilities.layoutCompoundLabel(this.tabPane, fm, text, icon, 0, 0, 0, 11, viewR, iconR, textR, this.textIconGap);
        this.tabPane.putClientProperty("html", null);
        int tabLabelShiftX = this.getTabLabelShiftX(n, n2, b);
        int tabLabelShiftY = this.getTabLabelShiftY(n, n2, b);
        iconR.x += tabLabelShiftX;
        iconR.y += tabLabelShiftY;
        textR.x += tabLabelShiftX;
        textR.y += tabLabelShiftY;
    }
    
    protected void paintIcon(Graphics graphics, int n, int n2, Icon icon, Rectangle rectangle, boolean b) {
        if (icon != null) {
            icon.paintIcon(this.tabPane, graphics, rectangle.x, rectangle.y);
        }
    }
    
    protected void paintText(Graphics g, int n, Font font, FontMetrics fontMetrics, int index, String text, Rectangle rectangle, boolean b) {
        g.setFont(font);
        View textViewForTab = this.getTextViewForTab(index);
        if (textViewForTab != null) {
            textViewForTab.paint(g, rectangle);
        }
        else {
            int displayedMnemonicIndex = this.tabPane.getDisplayedMnemonicIndexAt(index);
            if (this.tabPane.isEnabled() && this.tabPane.isEnabledAt(index)) {
                Color foreground = this.tabPane.getForegroundAt(index);
                if (b && foreground instanceof UIResource) {
                    Color color = UIManager.getColor("TabbedPane.selectedForeground");
                    if (color != null) {
                        foreground = color;
                    }
                }
                g.setColor(foreground);
                SwingUtilities2.drawStringUnderlineCharAt(this.tabPane, g, text, displayedMnemonicIndex, rectangle.x, rectangle.y + fontMetrics.getAscent());
            }
            else {
                g.setColor(this.tabPane.getBackgroundAt(index).brighter());
                SwingUtilities2.drawStringUnderlineCharAt(this.tabPane, g, text, displayedMnemonicIndex, rectangle.x, rectangle.y + fontMetrics.getAscent());
                g.setColor(this.tabPane.getBackgroundAt(index).darker());
                SwingUtilities2.drawStringUnderlineCharAt(this.tabPane, g, text, displayedMnemonicIndex, rectangle.x - 1, rectangle.y + fontMetrics.getAscent() - 1);
            }
        }
    }
    
    protected int getTabLabelShiftX(int n, int n2, boolean b) {
        Rectangle rectangle = this.rects[n2];
        int int1 = DefaultLookup.getInt(this.tabPane, this, "TabbedPane." + (b ? "selectedLabelShift" : "labelShift"), 1);
        switch (n) {
            case 2: {
                return int1;
            }
            case 4: {
                return -int1;
            }
            default: {
                return rectangle.width % 2;
            }
        }
    }
    
    protected int getTabLabelShiftY(int n, int n2, boolean b) {
        Rectangle rectangle = this.rects[n2];
        int n3 = b ? DefaultLookup.getInt(this.tabPane, this, "TabbedPane.selectedLabelShift", -1) : DefaultLookup.getInt(this.tabPane, this, "TabbedPane.labelShift", 1);
        switch (n) {
            case 3: {
                return -n3;
            }
            case 2:
            case 4: {
                return rectangle.height % 2;
            }
            default: {
                return n3;
            }
        }
    }
    
    protected void paintFocusIndicator(Graphics graphics, int n, Rectangle[] array, int n2, Rectangle rectangle, Rectangle rectangle2, boolean b) {
        Rectangle rectangle3 = array[n2];
        if (this.tabPane.hasFocus() && b) {
            graphics.setColor(this.focus);
            int n3;
            int n4;
            int n5;
            int n6;
            switch (n) {
                case 2: {
                    n3 = rectangle3.x + 3;
                    n4 = rectangle3.y + 3;
                    n5 = rectangle3.width - 5;
                    n6 = rectangle3.height - 6;
                    break;
                }
                case 4: {
                    n3 = rectangle3.x + 2;
                    n4 = rectangle3.y + 3;
                    n5 = rectangle3.width - 5;
                    n6 = rectangle3.height - 6;
                    break;
                }
                case 3: {
                    n3 = rectangle3.x + 3;
                    n4 = rectangle3.y + 2;
                    n5 = rectangle3.width - 6;
                    n6 = rectangle3.height - 5;
                    break;
                }
                default: {
                    n3 = rectangle3.x + 3;
                    n4 = rectangle3.y + 3;
                    n5 = rectangle3.width - 6;
                    n6 = rectangle3.height - 5;
                    break;
                }
            }
            BasicGraphicsUtils.drawDashedRect(graphics, n3, n4, n5, n6);
        }
    }
    
    protected void paintTabBorder(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        graphics.setColor(this.lightHighlight);
        switch (n) {
            case 2: {
                graphics.drawLine(n3 + 1, n4 + n6 - 2, n3 + 1, n4 + n6 - 2);
                graphics.drawLine(n3, n4 + 2, n3, n4 + n6 - 3);
                graphics.drawLine(n3 + 1, n4 + 1, n3 + 1, n4 + 1);
                graphics.drawLine(n3 + 2, n4, n3 + n5 - 1, n4);
                graphics.setColor(this.shadow);
                graphics.drawLine(n3 + 2, n4 + n6 - 2, n3 + n5 - 1, n4 + n6 - 2);
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n3 + 2, n4 + n6 - 1, n3 + n5 - 1, n4 + n6 - 1);
                break;
            }
            case 4: {
                graphics.drawLine(n3, n4, n3 + n5 - 3, n4);
                graphics.setColor(this.shadow);
                graphics.drawLine(n3, n4 + n6 - 2, n3 + n5 - 3, n4 + n6 - 2);
                graphics.drawLine(n3 + n5 - 2, n4 + 2, n3 + n5 - 2, n4 + n6 - 3);
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n3 + n5 - 2, n4 + 1, n3 + n5 - 2, n4 + 1);
                graphics.drawLine(n3 + n5 - 2, n4 + n6 - 2, n3 + n5 - 2, n4 + n6 - 2);
                graphics.drawLine(n3 + n5 - 1, n4 + 2, n3 + n5 - 1, n4 + n6 - 3);
                graphics.drawLine(n3, n4 + n6 - 1, n3 + n5 - 3, n4 + n6 - 1);
                break;
            }
            case 3: {
                graphics.drawLine(n3, n4, n3, n4 + n6 - 3);
                graphics.drawLine(n3 + 1, n4 + n6 - 2, n3 + 1, n4 + n6 - 2);
                graphics.setColor(this.shadow);
                graphics.drawLine(n3 + 2, n4 + n6 - 2, n3 + n5 - 3, n4 + n6 - 2);
                graphics.drawLine(n3 + n5 - 2, n4, n3 + n5 - 2, n4 + n6 - 3);
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n3 + 2, n4 + n6 - 1, n3 + n5 - 3, n4 + n6 - 1);
                graphics.drawLine(n3 + n5 - 2, n4 + n6 - 2, n3 + n5 - 2, n4 + n6 - 2);
                graphics.drawLine(n3 + n5 - 1, n4, n3 + n5 - 1, n4 + n6 - 3);
                break;
            }
            default: {
                graphics.drawLine(n3, n4 + 2, n3, n4 + n6 - 1);
                graphics.drawLine(n3 + 1, n4 + 1, n3 + 1, n4 + 1);
                graphics.drawLine(n3 + 2, n4, n3 + n5 - 3, n4);
                graphics.setColor(this.shadow);
                graphics.drawLine(n3 + n5 - 2, n4 + 2, n3 + n5 - 2, n4 + n6 - 1);
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n3 + n5 - 1, n4 + 2, n3 + n5 - 1, n4 + n6 - 1);
                graphics.drawLine(n3 + n5 - 2, n4 + 1, n3 + n5 - 2, n4 + 1);
                break;
            }
        }
    }
    
    protected void paintTabBackground(Graphics graphics, int n, int index, int n2, int n3, int n4, int n5, boolean b) {
        graphics.setColor((!b || this.selectedColor == null) ? this.tabPane.getBackgroundAt(index) : this.selectedColor);
        switch (n) {
            case 2: {
                graphics.fillRect(n2 + 1, n3 + 1, n4 - 1, n5 - 3);
                break;
            }
            case 4: {
                graphics.fillRect(n2, n3 + 1, n4 - 2, n5 - 3);
                break;
            }
            case 3: {
                graphics.fillRect(n2 + 1, n3, n4 - 3, n5 - 1);
                break;
            }
            default: {
                graphics.fillRect(n2 + 1, n3 + 1, n4 - 3, n5 - 1);
                break;
            }
        }
    }
    
    protected void paintContentBorder(Graphics graphics, int n, int n2) {
        int width = this.tabPane.getWidth();
        int height = this.tabPane.getHeight();
        Insets insets = this.tabPane.getInsets();
        Insets tabAreaInsets = this.getTabAreaInsets(n);
        int left = insets.left;
        int top = insets.top;
        int n3 = width - insets.right - insets.left;
        int n4 = height - insets.top - insets.bottom;
        switch (n) {
            case 2: {
                left += this.calculateTabAreaWidth(n, this.runCount, this.maxTabWidth);
                if (this.tabsOverlapBorder) {
                    left -= tabAreaInsets.right;
                }
                n3 -= left - insets.left;
                break;
            }
            case 4: {
                n3 -= this.calculateTabAreaWidth(n, this.runCount, this.maxTabWidth);
                if (this.tabsOverlapBorder) {
                    n3 += tabAreaInsets.left;
                    break;
                }
                break;
            }
            case 3: {
                n4 -= this.calculateTabAreaHeight(n, this.runCount, this.maxTabHeight);
                if (this.tabsOverlapBorder) {
                    n4 += tabAreaInsets.top;
                    break;
                }
                break;
            }
            default: {
                top += this.calculateTabAreaHeight(n, this.runCount, this.maxTabHeight);
                if (this.tabsOverlapBorder) {
                    top -= tabAreaInsets.bottom;
                }
                n4 -= top - insets.top;
                break;
            }
        }
        if (this.tabPane.getTabCount() > 0 && (this.contentOpaque || this.tabPane.isOpaque())) {
            Color color = UIManager.getColor("TabbedPane.contentAreaColor");
            if (color != null) {
                graphics.setColor(color);
            }
            else if (this.selectedColor == null || n2 == -1) {
                graphics.setColor(this.tabPane.getBackground());
            }
            else {
                graphics.setColor(this.selectedColor);
            }
            graphics.fillRect(left, top, n3, n4);
        }
        this.paintContentBorderTopEdge(graphics, n, n2, left, top, n3, n4);
        this.paintContentBorderLeftEdge(graphics, n, n2, left, top, n3, n4);
        this.paintContentBorderBottomEdge(graphics, n, n2, left, top, n3, n4);
        this.paintContentBorderRightEdge(graphics, n, n2, left, top, n3, n4);
    }
    
    protected void paintContentBorderTopEdge(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6) {
        Rectangle rectangle = (n2 < 0) ? null : this.getTabBounds(n2, this.calcRect);
        graphics.setColor(this.lightHighlight);
        if (n != 1 || n2 < 0 || rectangle.y + rectangle.height + 1 < n4 || rectangle.x < n3 || rectangle.x > n3 + n5) {
            graphics.drawLine(n3, n4, n3 + n5 - 2, n4);
        }
        else {
            graphics.drawLine(n3, n4, rectangle.x - 1, n4);
            if (rectangle.x + rectangle.width < n3 + n5 - 2) {
                graphics.drawLine(rectangle.x + rectangle.width, n4, n3 + n5 - 2, n4);
            }
            else {
                graphics.setColor(this.shadow);
                graphics.drawLine(n3 + n5 - 2, n4, n3 + n5 - 2, n4);
            }
        }
    }
    
    protected void paintContentBorderLeftEdge(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6) {
        Rectangle rectangle = (n2 < 0) ? null : this.getTabBounds(n2, this.calcRect);
        graphics.setColor(this.lightHighlight);
        if (n != 2 || n2 < 0 || rectangle.x + rectangle.width + 1 < n3 || rectangle.y < n4 || rectangle.y > n4 + n6) {
            graphics.drawLine(n3, n4, n3, n4 + n6 - 2);
        }
        else {
            graphics.drawLine(n3, n4, n3, rectangle.y - 1);
            if (rectangle.y + rectangle.height < n4 + n6 - 2) {
                graphics.drawLine(n3, rectangle.y + rectangle.height, n3, n4 + n6 - 2);
            }
        }
    }
    
    protected void paintContentBorderBottomEdge(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6) {
        Rectangle rectangle = (n2 < 0) ? null : this.getTabBounds(n2, this.calcRect);
        graphics.setColor(this.shadow);
        if (n != 3 || n2 < 0 || rectangle.y - 1 > n6 || rectangle.x < n3 || rectangle.x > n3 + n5) {
            graphics.drawLine(n3 + 1, n4 + n6 - 2, n3 + n5 - 2, n4 + n6 - 2);
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n3, n4 + n6 - 1, n3 + n5 - 1, n4 + n6 - 1);
        }
        else {
            graphics.drawLine(n3 + 1, n4 + n6 - 2, rectangle.x - 1, n4 + n6 - 2);
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n3, n4 + n6 - 1, rectangle.x - 1, n4 + n6 - 1);
            if (rectangle.x + rectangle.width < n3 + n5 - 2) {
                graphics.setColor(this.shadow);
                graphics.drawLine(rectangle.x + rectangle.width, n4 + n6 - 2, n3 + n5 - 2, n4 + n6 - 2);
                graphics.setColor(this.darkShadow);
                graphics.drawLine(rectangle.x + rectangle.width, n4 + n6 - 1, n3 + n5 - 1, n4 + n6 - 1);
            }
        }
    }
    
    protected void paintContentBorderRightEdge(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6) {
        Rectangle rectangle = (n2 < 0) ? null : this.getTabBounds(n2, this.calcRect);
        graphics.setColor(this.shadow);
        if (n != 4 || n2 < 0 || rectangle.x - 1 > n5 || rectangle.y < n4 || rectangle.y > n4 + n6) {
            graphics.drawLine(n3 + n5 - 2, n4 + 1, n3 + n5 - 2, n4 + n6 - 3);
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n3 + n5 - 1, n4, n3 + n5 - 1, n4 + n6 - 1);
        }
        else {
            graphics.drawLine(n3 + n5 - 2, n4 + 1, n3 + n5 - 2, rectangle.y - 1);
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n3 + n5 - 1, n4, n3 + n5 - 1, rectangle.y - 1);
            if (rectangle.y + rectangle.height < n4 + n6 - 2) {
                graphics.setColor(this.shadow);
                graphics.drawLine(n3 + n5 - 2, rectangle.y + rectangle.height, n3 + n5 - 2, n4 + n6 - 2);
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n3 + n5 - 1, rectangle.y + rectangle.height, n3 + n5 - 1, n4 + n6 - 2);
            }
        }
    }
    
    private void ensureCurrentLayout() {
        if (!this.tabPane.isValid()) {
            this.tabPane.validate();
        }
        if (!this.tabPane.isValid()) {
            ((BasicTabbedPaneUI.TabbedPaneLayout)this.tabPane.getLayout()).calculateLayoutInfo();
        }
    }
    
    @Override
    public Rectangle getTabBounds(JTabbedPane tabbedPane, int n) {
        this.ensureCurrentLayout();
        return this.getTabBounds(n, new Rectangle());
    }
    
    @Override
    public int getTabRunCount(JTabbedPane tabbedPane) {
        this.ensureCurrentLayout();
        return this.runCount;
    }
    
    @Override
    public int tabForCoordinate(JTabbedPane tabbedPane, int n, int n2) {
        return this.tabForCoordinate(tabbedPane, n, n2, true);
    }
    
    private int tabForCoordinate(JTabbedPane tabbedPane, int x, int y, boolean b) {
        if (b) {
            this.ensureCurrentLayout();
        }
        if (this.isRunsDirty) {
            return -1;
        }
        Point p4 = new Point(x, y);
        if (this.scrollableTabLayoutEnabled()) {
            this.translatePointToTabPanel(x, y, p4);
            if (!this.tabScroller.viewport.getViewRect().contains(p4)) {
                return -1;
            }
        }
        int tabCount = this.tabPane.getTabCount();
        int i = 0;
        while (i < tabCount) {
            if (this.rects[i].contains(p4.x, p4.y)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    protected Rectangle getTabBounds(int n, Rectangle rectangle) {
        rectangle.width = this.rects[n].width;
        rectangle.height = this.rects[n].height;
        if (this.scrollableTabLayoutEnabled()) {
            Point location = this.tabScroller.viewport.getLocation();
            Point viewPosition = this.tabScroller.viewport.getViewPosition();
            rectangle.x = this.rects[n].x + location.x - viewPosition.x;
            rectangle.y = this.rects[n].y + location.y - viewPosition.y;
            return rectangle;
        }
        rectangle.x = this.rects[n].x;
        rectangle.y = this.rects[n].y;
        return rectangle;
    }
    
    private int getClosestTab(int n, int n2) {
        int i = 0;
        int min = Math.min(this.rects.length, this.tabPane.getTabCount());
        int tabPlacement = this.tabPane.getTabPlacement();
        int n3 = tabPlacement == 1 || tabPlacement == 3;
        n2 = (n3 ? n : n2);
        while (i != min) {
            int n4 = (min + i) / 2;
            int n5;
            int n6;
            if (!(n3 == 0)) {
                n5 = this.rects[n4].x;
                n6 = n5 + this.rects[n4].width;
            }
            else {
                n5 = this.rects[n4].y;
                n6 = n5 + this.rects[n4].height;
            }
            if (n2 < n5) {
                min = n4;
                if (i == min) {
                    return Math.max(0, n4 - 1);
                }
                continue;
            }
            else if (n2 >= n6) {
                i = n4;
                if (min - i <= 1) {
                    return Math.max(n4 + 1, min - 1);
                }
                continue;
            }
            else {
                return n4;
            }
        }
        return i;
    }
    
    private Point translatePointToTabPanel(int n, int n2, Point point) {
        Point location = this.tabScroller.viewport.getLocation();
        Point viewPosition = this.tabScroller.viewport.getViewPosition();
        point.x = n - location.x + viewPosition.x;
        point.y = n2 - location.y + viewPosition.y;
        return point;
    }
    
    protected Component getVisibleComponent() {
        return this.visibleComponent;
    }
    
    protected void setVisibleComponent(Component visibleComponent) {
        if (this.visibleComponent != null && this.visibleComponent != visibleComponent && this.visibleComponent.getParent() == this.tabPane && this.visibleComponent.isVisible()) {
            this.visibleComponent.setVisible(false);
        }
        if (visibleComponent != null && !visibleComponent.isVisible()) {
            visibleComponent.setVisible(true);
        }
        this.visibleComponent = visibleComponent;
    }
    
    protected void assureRectsCreated(int b) {
        int i = this.rects.length;
        if (b != i) {
            Rectangle[] rects = new Rectangle[b];
            System.arraycopy(this.rects, 0, rects, 0, Math.min(i, b));
            this.rects = rects;
            for (; i < b; ++i) {
                this.rects[i] = new Rectangle();
            }
        }
    }
    
    protected void expandTabRunsArray() {
        int[] tabRuns = new int[this.tabRuns.length + 10];
        System.arraycopy(this.tabRuns, 0, tabRuns, 0, this.runCount);
        this.tabRuns = tabRuns;
    }
    
    protected int getRunForTab(int n, int n2) {
        int i = 0;
        while (i < this.runCount) {
            int n3 = this.tabRuns[i];
            int lastTabInRun = this.lastTabInRun(n, i);
            if (n2 >= n3 && n2 <= lastTabInRun) {
                return i;
            }
            ++i;
        }
        return 0;
    }
    
    protected int lastTabInRun(int n, int n2) {
        if (this.runCount == 1) {
            return n - 1;
        }
        int n3 = (n2 == this.runCount - 1) ? 0 : (n2 + 1);
        if (this.tabRuns[n3] == 0) {
            return n - 1;
        }
        return this.tabRuns[n3] - 1;
    }
    
    protected int getTabRunOverlay(int n) {
        return this.tabRunOverlay;
    }
    
    protected int getTabRunIndent(int n, int n2) {
        return 0;
    }
    
    protected boolean shouldPadTabRun(int n, int n2) {
        return this.runCount > 1;
    }
    
    protected boolean shouldRotateTabRuns(int n) {
        return true;
    }
    
    protected Icon getIconForTab(int index) {
        return (!this.tabPane.isEnabled() || !this.tabPane.isEnabledAt(index)) ? this.tabPane.getDisabledIconAt(index) : this.tabPane.getIconAt(index);
    }
    
    protected View getTextViewForTab(int index) {
        if (this.htmlViews != null) {
            return (View)this.htmlViews.elementAt(index);
        }
        return null;
    }
    
    protected int calculateTabHeight(int n, int index, int n2) {
        int n3 = 0;
        Component tabComponent = this.tabPane.getTabComponentAt(index);
        int a;
        if (tabComponent != null) {
            a = tabComponent.getPreferredSize().height;
        }
        else {
            View textViewForTab = this.getTextViewForTab(index);
            if (textViewForTab != null) {
                a = n3 + (int)textViewForTab.getPreferredSpan(1);
            }
            else {
                a = n3 + n2;
            }
            Icon iconForTab = this.getIconForTab(index);
            if (iconForTab != null) {
                a = Math.max(a, iconForTab.getIconHeight());
            }
        }
        Insets tabInsets = this.getTabInsets(n, index);
        return a + (tabInsets.top + tabInsets.bottom + 2);
    }
    
    protected int calculateMaxTabHeight(int n) {
        FontMetrics fontMetrics = this.getFontMetrics();
        int tabCount = this.tabPane.getTabCount();
        int max = 0;
        int height = fontMetrics.getHeight();
        for (int i = 0; i < tabCount; ++i) {
            max = Math.max(this.calculateTabHeight(n, i, height), max);
        }
        return max;
    }
    
    protected int calculateTabWidth(int n, int n2, FontMetrics fm) {
        Insets tabInsets = this.getTabInsets(n, n2);
        int n3 = tabInsets.left + tabInsets.right + 3;
        Component tabComponent = this.tabPane.getTabComponentAt(n2);
        int n4;
        if (tabComponent != null) {
            n4 = n3 + tabComponent.getPreferredSize().width;
        }
        else {
            Icon iconForTab = this.getIconForTab(n2);
            if (iconForTab != null) {
                n3 += iconForTab.getIconWidth() + this.textIconGap;
            }
            View textViewForTab = this.getTextViewForTab(n2);
            if (textViewForTab != null) {
                n4 = n3 + (int)textViewForTab.getPreferredSpan(0);
            }
            else {
                n4 = n3 + SwingUtilities2.stringWidth(this.tabPane, fm, this.tabPane.getTitleAt(n2));
            }
        }
        return n4;
    }
    
    protected int calculateMaxTabWidth(int n) {
        FontMetrics fontMetrics = this.getFontMetrics();
        int tabCount = this.tabPane.getTabCount();
        int max = 0;
        for (int i = 0; i < tabCount; ++i) {
            max = Math.max(this.calculateTabWidth(n, i, fontMetrics), max);
        }
        return max;
    }
    
    protected int calculateTabAreaHeight(int n, int n2, int n3) {
        Insets tabAreaInsets = this.getTabAreaInsets(n);
        int tabRunOverlay = this.getTabRunOverlay(n);
        return (n2 > 0) ? (n2 * (n3 - tabRunOverlay) + tabRunOverlay + tabAreaInsets.top + tabAreaInsets.bottom) : 0;
    }
    
    protected int calculateTabAreaWidth(int n, int n2, int n3) {
        Insets tabAreaInsets = this.getTabAreaInsets(n);
        int tabRunOverlay = this.getTabRunOverlay(n);
        return (n2 > 0) ? (n2 * (n3 - tabRunOverlay) + tabRunOverlay + tabAreaInsets.left + tabAreaInsets.right) : 0;
    }
    
    protected Insets getTabInsets(int n, int n2) {
        return this.tabInsets;
    }
    
    protected Insets getSelectedTabPadInsets(int n) {
        rotateInsets(this.selectedTabPadInsets, this.currentPadInsets, n);
        return this.currentPadInsets;
    }
    
    protected Insets getTabAreaInsets(int n) {
        rotateInsets(this.tabAreaInsets, this.currentTabAreaInsets, n);
        return this.currentTabAreaInsets;
    }
    
    protected Insets getContentBorderInsets(int n) {
        return this.contentBorderInsets;
    }
    
    protected FontMetrics getFontMetrics() {
        return this.tabPane.getFontMetrics(this.tabPane.getFont());
    }
    
    protected void navigateSelectedTab(int n) {
        int tabPlacement = this.tabPane.getTabPlacement();
        int n2 = DefaultLookup.getBoolean(this.tabPane, this, "TabbedPane.selectionFollowsFocus", true) ? this.tabPane.getSelectedIndex() : this.getFocusIndex();
        int tabCount = this.tabPane.getTabCount();
        int leftToRight = BasicGraphicsUtils.isLeftToRight(this.tabPane);
        if (tabCount <= 0) {
            return;
        }
        switch (tabPlacement) {
            case 2:
            case 4: {
                switch (n) {
                    case 12: {
                        this.selectNextTab(n2);
                        break;
                    }
                    case 13: {
                        this.selectPreviousTab(n2);
                        break;
                    }
                    case 1: {
                        this.selectPreviousTabInRun(n2);
                        break;
                    }
                    case 5: {
                        this.selectNextTabInRun(n2);
                        break;
                    }
                    case 7: {
                        this.selectAdjacentRunTab(tabPlacement, n2, this.getTabRunOffset(tabPlacement, tabCount, n2, false));
                        break;
                    }
                    case 3: {
                        this.selectAdjacentRunTab(tabPlacement, n2, this.getTabRunOffset(tabPlacement, tabCount, n2, true));
                        break;
                    }
                    default: {
                        break;
                    }
                }
                break;
            }
            default: {
                switch (n) {
                    case 12: {
                        this.selectNextTab(n2);
                        break;
                    }
                    case 13: {
                        this.selectPreviousTab(n2);
                        break;
                    }
                    case 1: {
                        this.selectAdjacentRunTab(tabPlacement, n2, this.getTabRunOffset(tabPlacement, tabCount, n2, false));
                        break;
                    }
                    case 5: {
                        this.selectAdjacentRunTab(tabPlacement, n2, this.getTabRunOffset(tabPlacement, tabCount, n2, true));
                        break;
                    }
                    case 3: {
                        if (!(leftToRight == 0)) {
                            this.selectNextTabInRun(n2);
                            break;
                        }
                        this.selectPreviousTabInRun(n2);
                        break;
                    }
                    case 7: {
                        if (!(leftToRight == 0)) {
                            this.selectPreviousTabInRun(n2);
                            break;
                        }
                        this.selectNextTabInRun(n2);
                        break;
                    }
                }
                break;
            }
        }
        return;
    }
    
    protected void selectNextTabInRun(int n) {
        int tabCount;
        int index;
        for (tabCount = this.tabPane.getTabCount(), index = this.getNextTabIndexInRun(tabCount, n); index != n && !this.tabPane.isEnabledAt(index); index = this.getNextTabIndexInRun(tabCount, index)) {}
        this.navigateTo(index);
    }
    
    protected void selectPreviousTabInRun(int n) {
        int tabCount;
        int index;
        for (tabCount = this.tabPane.getTabCount(), index = this.getPreviousTabIndexInRun(tabCount, n); index != n && !this.tabPane.isEnabledAt(index); index = this.getPreviousTabIndexInRun(tabCount, index)) {}
        this.navigateTo(index);
    }
    
    protected void selectNextTab(int n) {
        int index;
        for (index = this.getNextTabIndex(n); index != n && !this.tabPane.isEnabledAt(index); index = this.getNextTabIndex(index)) {}
        this.navigateTo(index);
    }
    
    protected void selectPreviousTab(int n) {
        int index;
        for (index = this.getPreviousTabIndex(n); index != n && !this.tabPane.isEnabledAt(index); index = this.getPreviousTabIndex(index)) {}
        this.navigateTo(index);
    }
    
    protected void selectAdjacentRunTab(int n, int n2, int n3) {
        if (this.runCount < 2) {
            return;
        }
        Rectangle rectangle = this.rects[n2];
        int index;
        switch (n) {
            case 2:
            case 4: {
                index = this.tabForCoordinate(this.tabPane, rectangle.x + rectangle.width / 2 + n3, rectangle.y + rectangle.height / 2);
                break;
            }
            default: {
                index = this.tabForCoordinate(this.tabPane, rectangle.x + rectangle.width / 2, rectangle.y + rectangle.height / 2 + n3);
                break;
            }
        }
        if (index != -1) {
            while (!this.tabPane.isEnabledAt(index) && index != n2) {
                index = this.getNextTabIndex(index);
            }
            this.navigateTo(index);
        }
        return;
    }
    
    private void navigateTo(int selectedIndex) {
        if (DefaultLookup.getBoolean(this.tabPane, this, "TabbedPane.selectionFollowsFocus", true)) {
            this.tabPane.setSelectedIndex(selectedIndex);
        }
        else {
            this.setFocusIndex(selectedIndex, true);
        }
    }
    
    void setFocusIndex(int n, boolean b) {
        if (b && !this.isRunsDirty) {
            this.repaintTab(this.focusIndex);
            this.repaintTab(this.focusIndex = n);
        }
        else {
            this.focusIndex = n;
        }
    }
    
    private void repaintTab(int n) {
        if (!this.isRunsDirty && n >= 0 && n < this.tabPane.getTabCount()) {
            this.tabPane.repaint(this.getTabBounds(this.tabPane, n));
        }
    }
    
    private void validateFocusIndex() {
        if (this.focusIndex >= this.tabPane.getTabCount()) {
            this.setFocusIndex(this.tabPane.getSelectedIndex(), false);
        }
    }
    
    protected int getFocusIndex() {
        return this.focusIndex;
    }
    
    protected int getTabRunOffset(int n, int n2, int n3, boolean b) {
        int runForTab = this.getRunForTab(n2, n3);
        int n4;
        switch (n) {
            case 2: {
                if (runForTab == 0) {
                    n4 = (b ? (-(this.calculateTabAreaWidth(n, this.runCount, this.maxTabWidth) - this.maxTabWidth)) : (-this.maxTabWidth));
                    break;
                }
                if (runForTab == this.runCount - 1) {
                    n4 = (b ? this.maxTabWidth : (this.calculateTabAreaWidth(n, this.runCount, this.maxTabWidth) - this.maxTabWidth));
                    break;
                }
                n4 = (b ? this.maxTabWidth : (-this.maxTabWidth));
                break;
            }
            case 4: {
                if (runForTab == 0) {
                    n4 = (b ? this.maxTabWidth : (this.calculateTabAreaWidth(n, this.runCount, this.maxTabWidth) - this.maxTabWidth));
                    break;
                }
                if (runForTab == this.runCount - 1) {
                    n4 = (b ? (-(this.calculateTabAreaWidth(n, this.runCount, this.maxTabWidth) - this.maxTabWidth)) : (-this.maxTabWidth));
                    break;
                }
                n4 = (b ? this.maxTabWidth : (-this.maxTabWidth));
                break;
            }
            case 3: {
                if (runForTab == 0) {
                    n4 = (b ? this.maxTabHeight : (this.calculateTabAreaHeight(n, this.runCount, this.maxTabHeight) - this.maxTabHeight));
                    break;
                }
                if (runForTab == this.runCount - 1) {
                    n4 = (b ? (-(this.calculateTabAreaHeight(n, this.runCount, this.maxTabHeight) - this.maxTabHeight)) : (-this.maxTabHeight));
                    break;
                }
                n4 = (b ? this.maxTabHeight : (-this.maxTabHeight));
                break;
            }
            default: {
                if (runForTab == 0) {
                    n4 = (b ? (-(this.calculateTabAreaHeight(n, this.runCount, this.maxTabHeight) - this.maxTabHeight)) : (-this.maxTabHeight));
                    break;
                }
                if (runForTab == this.runCount - 1) {
                    n4 = (b ? this.maxTabHeight : (this.calculateTabAreaHeight(n, this.runCount, this.maxTabHeight) - this.maxTabHeight));
                    break;
                }
                n4 = (b ? this.maxTabHeight : (-this.maxTabHeight));
                break;
            }
        }
        return n4;
    }
    
    protected int getPreviousTabIndex(int n) {
        int n2 = (n - 1 >= 0) ? (n - 1) : (this.tabPane.getTabCount() - 1);
        return (n2 >= 0) ? n2 : false;
    }
    
    protected int getNextTabIndex(int n) {
        return (n + 1) % this.tabPane.getTabCount();
    }
    
    protected int getNextTabIndexInRun(int n, int n2) {
        if (this.runCount < 2) {
            return this.getNextTabIndex(n2);
        }
        int runForTab = this.getRunForTab(n, n2);
        int nextTabIndex = this.getNextTabIndex(n2);
        if (nextTabIndex == this.tabRuns[this.getNextTabRun(runForTab)]) {
            return this.tabRuns[runForTab];
        }
        return nextTabIndex;
    }
    
    protected int getPreviousTabIndexInRun(int n, int n2) {
        if (this.runCount < 2) {
            return this.getPreviousTabIndex(n2);
        }
        int runForTab = this.getRunForTab(n, n2);
        if (n2 == this.tabRuns[runForTab]) {
            int n3 = this.tabRuns[this.getNextTabRun(runForTab)] - 1;
            return (n3 != -1) ? n3 : (n - 1);
        }
        return this.getPreviousTabIndex(n2);
    }
    
    protected int getPreviousTabRun(int n) {
        int n2 = (n - 1 >= 0) ? (n - 1) : (this.runCount - 1);
        return (n2 >= 0) ? n2 : false;
    }
    
    protected int getNextTabRun(int n) {
        return (n + 1) % this.runCount;
    }
    
    protected static void rotateInsets(Insets insets, Insets insets2, int n) {
        switch (n) {
            case 2: {
                insets2.top = insets.left;
                insets2.left = insets.top;
                insets2.bottom = insets.right;
                insets2.right = insets.bottom;
                break;
            }
            case 3: {
                insets2.top = insets.bottom;
                insets2.left = insets.left;
                insets2.bottom = insets.top;
                insets2.right = insets.right;
                break;
            }
            case 4: {
                insets2.top = insets.left;
                insets2.left = insets.bottom;
                insets2.bottom = insets.right;
                insets2.right = insets.top;
                break;
            }
            default: {
                insets2.top = insets.top;
                insets2.left = insets.left;
                insets2.bottom = insets.bottom;
                insets2.right = insets.right;
                break;
            }
        }
    }
    
    boolean requestFocusForVisibleComponent() {
        return SwingUtilities2.tabbedPaneChangeFocusTo(this.getVisibleComponent());
    }
    
    private Vector<View> createHTMLVector() {
        Vector<View> vector = new Vector();
        int tabCount = this.tabPane.getTabCount();
        if (tabCount > 0) {
            for (int i = 0; i < tabCount; ++i) {
                String title = this.tabPane.getTitleAt(i);
                if (BasicHTML.isHTMLString(title)) {
                    vector.addElement(BasicHTML.createHTMLView(this.tabPane, title));
                }
                else {
                    vector.addElement(null);
                }
            }
            return vector;
        }
        return vector;
    }
    
    static boolean access$400(BasicTabbedPaneUI basicTabbedPaneUI) {
        return basicTabbedPaneUI.scrollableTabLayoutEnabled();
    }
    
    static Point access$800(BasicTabbedPaneUI basicTabbedPaneUI, int n, int n2, Point point) {
        return basicTabbedPaneUI.translatePointToTabPanel(n, n2, point);
    }
    
    static int access$1200(BasicTabbedPaneUI basicTabbedPaneUI, int n, int n2) {
        return basicTabbedPaneUI.getClosestTab(n, n2);
    }
    
    static void access$1300(BasicTabbedPaneUI basicTabbedPaneUI) {
        basicTabbedPaneUI.updateMnemonics();
    }
    
    static void access$1800(BasicTabbedPaneUI basicTabbedPaneUI) {
        basicTabbedPaneUI.installTabContainer();
    }
    
    static Vector access$1900(BasicTabbedPaneUI basicTabbedPaneUI) {
        return basicTabbedPaneUI.createHTMLVector();
    }
    
    static void access$2000(BasicTabbedPaneUI basicTabbedPaneUI) {
        basicTabbedPaneUI.ensureCurrentLayout();
    }
    
    static void access$2100(BasicTabbedPaneUI basicTabbedPaneUI, int n, int n2) {
        basicTabbedPaneUI.setRolloverTab(n, n2);
    }
    
    static void access$2300(BasicTabbedPaneUI basicTabbedPaneUI, int n) {
        basicTabbedPaneUI.repaintTab(n);
    }
    
    static void access$2400(BasicTabbedPaneUI basicTabbedPaneUI) {
        basicTabbedPaneUI.validateFocusIndex();
    }
    
    static BasicTabbedPaneUI.Handler access$2500(BasicTabbedPaneUI basicTabbedPaneUI) {
        return basicTabbedPaneUI.getHandler();
    }
    
    static Polygon access$2700(int n, Rectangle rectangle, int n2) {
        return createCroppedTabShape(n, rectangle, n2);
    }
    
    static void access$2800(BasicTabbedPaneUI basicTabbedPaneUI, Graphics graphics) {
        basicTabbedPaneUI.paintCroppedTabEdge(graphics);
    }
    
    static {
        BasicTabbedPaneUI.xCropLen = new int[] { 1, 1, 0, 0, 1, 1, 2, 2 };
        BasicTabbedPaneUI.yCropLen = new int[] { 0, 3, 3, 6, 6, 9, 9, 12 };
    }
    
    private static class Actions extends UIAction
    {
        static final String NEXT = "navigateNext";
        static final String PREVIOUS = "navigatePrevious";
        static final String RIGHT = "navigateRight";
        static final String LEFT = "navigateLeft";
        static final String UP = "navigateUp";
        static final String DOWN = "navigateDown";
        static final String PAGE_UP = "navigatePageUp";
        static final String PAGE_DOWN = "navigatePageDown";
        static final String REQUEST_FOCUS = "requestFocus";
        static final String REQUEST_FOCUS_FOR_VISIBLE = "requestFocusForVisibleComponent";
        static final String SET_SELECTED = "setSelectedIndex";
        static final String SELECT_FOCUSED = "selectTabWithFocus";
        static final String SCROLL_FORWARD = "scrollTabsForwardAction";
        static final String SCROLL_BACKWARD = "scrollTabsBackwardAction";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            String name = this.getName();
            JTabbedPane tabbedPane = (JTabbedPane)actionEvent.getSource();
            BasicTabbedPaneUI basicTabbedPaneUI = (BasicTabbedPaneUI)BasicLookAndFeel.getUIOfType(tabbedPane.getUI(), BasicTabbedPaneUI.class);
            if (basicTabbedPaneUI == null) {
                return;
            }
            if (name == "navigateNext") {
                basicTabbedPaneUI.navigateSelectedTab(12);
            }
            else if (name == "navigatePrevious") {
                basicTabbedPaneUI.navigateSelectedTab(13);
            }
            else if (name == "navigateRight") {
                basicTabbedPaneUI.navigateSelectedTab(3);
            }
            else if (name == "navigateLeft") {
                basicTabbedPaneUI.navigateSelectedTab(7);
            }
            else if (name == "navigateUp") {
                basicTabbedPaneUI.navigateSelectedTab(1);
            }
            else if (name == "navigateDown") {
                basicTabbedPaneUI.navigateSelectedTab(5);
            }
            else if (name == "navigatePageUp") {
                int tabPlacement2 = tabbedPane.getTabPlacement();
                if (tabPlacement2 == 1 || tabPlacement2 == 3) {
                    basicTabbedPaneUI.navigateSelectedTab(7);
                }
                else {
                    basicTabbedPaneUI.navigateSelectedTab(1);
                }
            }
            else if (name == "navigatePageDown") {
                int tabPlacement = tabbedPane.getTabPlacement();
                if (tabPlacement == 1 || tabPlacement == 3) {
                    basicTabbedPaneUI.navigateSelectedTab(3);
                }
                else {
                    basicTabbedPaneUI.navigateSelectedTab(5);
                }
            }
            else if (name == "requestFocus") {
                tabbedPane.requestFocus();
            }
            else if (name == "requestFocusForVisibleComponent") {
                basicTabbedPaneUI.requestFocusForVisibleComponent();
            }
            else if (name == "setSelectedIndex") {
                String actionCommand = actionEvent.getActionCommand();
                if (actionCommand != null && actionCommand.length() > 0) {
                    int char1 = actionEvent.getActionCommand().charAt(0);
                    if (char1 >= 97 && char1 <= 122) {
                        char1 -= 32;
                    }
                    Integer n = (Integer)basicTabbedPaneUI.mnemonicToIndexMap.get(Integer.valueOf(char1));
                    if (n != null && tabbedPane.isEnabledAt((int)n)) {
                        tabbedPane.setSelectedIndex((int)n);
                    }
                }
            }
            else if (name == "selectTabWithFocus") {
                int focusIndex = basicTabbedPaneUI.getFocusIndex();
                if (focusIndex != -1) {
                    tabbedPane.setSelectedIndex(focusIndex);
                }
            }
            else if (name == "scrollTabsForwardAction") {
                if (BasicTabbedPaneUI.access$400(basicTabbedPaneUI)) {
                    basicTabbedPaneUI.tabScroller.scrollForward(tabbedPane.getTabPlacement());
                }
            }
            else if (name == "scrollTabsBackwardAction" && BasicTabbedPaneUI.access$400(basicTabbedPaneUI)) {
                basicTabbedPaneUI.tabScroller.scrollBackward(tabbedPane.getTabPlacement());
            }
            return;
        }
    }
    
    public class TabbedPaneLayout implements LayoutManager
    {
        @Override
        public void addLayoutComponent(String s, Component component) {
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            return this.calculateSize(false);
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            return this.calculateSize(true);
        }
        
        protected Dimension calculateSize(boolean b) {
            int tabPlacement = BasicTabbedPaneUI.this.tabPane.getTabPlacement();
            Insets insets = BasicTabbedPaneUI.this.tabPane.getInsets();
            Insets contentBorderInsets = BasicTabbedPaneUI.this.getContentBorderInsets(tabPlacement);
            Insets tabAreaInsets = BasicTabbedPaneUI.this.getTabAreaInsets(tabPlacement);
            Dimension dimension = new Dimension(0, 0);
            int n = 0;
            int n2 = 0;
            int max = 0;
            int max2 = 0;
            for (int i = 0; i < BasicTabbedPaneUI.this.tabPane.getTabCount(); ++i) {
                Component component = BasicTabbedPaneUI.this.tabPane.getComponentAt(i);
                if (component != null) {
                    Dimension dimension2 = b ? component.getMinimumSize() : component.getPreferredSize();
                    if (dimension2 != null) {
                        max2 = Math.max(dimension2.height, max2);
                        max = Math.max(dimension2.width, max);
                    }
                }
            }
            int a = n2 + max;
            int a2 = n + max2;
            int max3;
            int max4;
            switch (tabPlacement) {
                case 2:
                case 4: {
                    max3 = Math.max(a2, BasicTabbedPaneUI.this.calculateMaxTabHeight(tabPlacement));
                    max4 = a + this.preferredTabAreaWidth(tabPlacement, max3 - tabAreaInsets.top - tabAreaInsets.bottom);
                    break;
                }
                default: {
                    max4 = Math.max(a, BasicTabbedPaneUI.this.calculateMaxTabWidth(tabPlacement));
                    max3 = a2 + this.preferredTabAreaHeight(tabPlacement, max4 - tabAreaInsets.left - tabAreaInsets.right);
                    break;
                }
            }
            return new Dimension(max4 + insets.left + insets.right + contentBorderInsets.left + contentBorderInsets.right, max3 + insets.bottom + insets.top + contentBorderInsets.top + contentBorderInsets.bottom);
        }
        
        protected int preferredTabAreaHeight(int n, int n2) {
            FontMetrics fontMetrics = BasicTabbedPaneUI.this.getFontMetrics();
            int tabCount = BasicTabbedPaneUI.this.tabPane.getTabCount();
            int calculateTabAreaHeight = 0;
            if (tabCount > 0) {
                int n3 = 1;
                int n4 = 0;
                int calculateMaxTabHeight = BasicTabbedPaneUI.this.calculateMaxTabHeight(n);
                int calculateTabWidth;
                for (int i = 0; i < tabCount; n4 += calculateTabWidth, ++i) {
                    calculateTabWidth = BasicTabbedPaneUI.this.calculateTabWidth(n, i, fontMetrics);
                    if (n4 != 0 && n4 + calculateTabWidth > n2) {
                        ++n3;
                        n4 = 0;
                    }
                }
                calculateTabAreaHeight = BasicTabbedPaneUI.this.calculateTabAreaHeight(n, n3, calculateMaxTabHeight);
            }
            return calculateTabAreaHeight;
        }
        
        protected int preferredTabAreaWidth(int n, int n2) {
            FontMetrics fontMetrics = BasicTabbedPaneUI.this.getFontMetrics();
            int tabCount = BasicTabbedPaneUI.this.tabPane.getTabCount();
            int calculateTabAreaWidth = 0;
            if (tabCount > 0) {
                int n3 = 1;
                int n4 = 0;
                int height = fontMetrics.getHeight();
                BasicTabbedPaneUI.this.maxTabWidth = BasicTabbedPaneUI.this.calculateMaxTabWidth(n);
                int calculateTabHeight;
                for (int i = 0; i < tabCount; n4 += calculateTabHeight, ++i) {
                    calculateTabHeight = BasicTabbedPaneUI.this.calculateTabHeight(n, i, height);
                    if (n4 != 0 && n4 + calculateTabHeight > n2) {
                        ++n3;
                        n4 = 0;
                    }
                }
                calculateTabAreaWidth = BasicTabbedPaneUI.this.calculateTabAreaWidth(n, n3, BasicTabbedPaneUI.this.maxTabWidth);
            }
            return calculateTabAreaWidth;
        }
        
        @Override
        public void layoutContainer(Container container) {
            BasicTabbedPaneUI.this.setRolloverTab(-1);
            int tabPlacement = BasicTabbedPaneUI.this.tabPane.getTabPlacement();
            Insets insets = BasicTabbedPaneUI.this.tabPane.getInsets();
            int selectedIndex = BasicTabbedPaneUI.this.tabPane.getSelectedIndex();
            Component visibleComponent = BasicTabbedPaneUI.this.getVisibleComponent();
            this.calculateLayoutInfo();
            Component component = null;
            if (selectedIndex < 0) {
                if (visibleComponent != null) {
                    BasicTabbedPaneUI.this.setVisibleComponent(null);
                }
            }
            else {
                component = BasicTabbedPaneUI.this.tabPane.getComponentAt(selectedIndex);
            }
            int n = 0;
            int n2 = 0;
            Insets contentBorderInsets = BasicTabbedPaneUI.this.getContentBorderInsets(tabPlacement);
            boolean n3 = false;
            if (component != null) {
                if (component != visibleComponent && visibleComponent != null && SwingUtilities.findFocusOwner(visibleComponent) != null) {
                    n3 = true;
                }
                BasicTabbedPaneUI.this.setVisibleComponent(component);
            }
            Rectangle bounds = BasicTabbedPaneUI.this.tabPane.getBounds();
            int componentCount = BasicTabbedPaneUI.this.tabPane.getComponentCount();
            if (componentCount > 0) {
                int x;
                int y;
                switch (tabPlacement) {
                    case 2: {
                        n = BasicTabbedPaneUI.this.calculateTabAreaWidth(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabWidth);
                        x = insets.left + n + contentBorderInsets.left;
                        y = insets.top + contentBorderInsets.top;
                        break;
                    }
                    case 4: {
                        n = BasicTabbedPaneUI.this.calculateTabAreaWidth(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabWidth);
                        x = insets.left + contentBorderInsets.left;
                        y = insets.top + contentBorderInsets.top;
                        break;
                    }
                    case 3: {
                        n2 = BasicTabbedPaneUI.this.calculateTabAreaHeight(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabHeight);
                        x = insets.left + contentBorderInsets.left;
                        y = insets.top + contentBorderInsets.top;
                        break;
                    }
                    default: {
                        n2 = BasicTabbedPaneUI.this.calculateTabAreaHeight(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabHeight);
                        x = insets.left + contentBorderInsets.left;
                        y = insets.top + n2 + contentBorderInsets.top;
                        break;
                    }
                }
                int width = bounds.width - n - insets.left - insets.right - contentBorderInsets.left - contentBorderInsets.right;
                int height = bounds.height - n2 - insets.top - insets.bottom - contentBorderInsets.top - contentBorderInsets.bottom;
                for (int i = 0; i < componentCount; ++i) {
                    Component component2 = BasicTabbedPaneUI.this.tabPane.getComponent(i);
                    if (component2 == BasicTabbedPaneUI.this.tabContainer) {
                        int width2 = (n == 0) ? bounds.width : (n + insets.left + insets.right + contentBorderInsets.left + contentBorderInsets.right);
                        int height2 = (n2 == 0) ? bounds.height : (n2 + insets.top + insets.bottom + contentBorderInsets.top + contentBorderInsets.bottom);
                        int x2 = 0;
                        int y2 = 0;
                        if (tabPlacement == 3) {
                            y2 = bounds.height - height2;
                        }
                        else if (tabPlacement == 4) {
                            x2 = bounds.width - width2;
                        }
                        component2.setBounds(x2, y2, width2, height2);
                    }
                    else {
                        component2.setBounds(x, y, width, height);
                    }
                }
            }
            this.layoutTabComponents();
            if (n3 && !BasicTabbedPaneUI.this.requestFocusForVisibleComponent()) {
                BasicTabbedPaneUI.this.tabPane.requestFocus();
            }
        }
        
        public void calculateLayoutInfo() {
            int tabCount = BasicTabbedPaneUI.this.tabPane.getTabCount();
            BasicTabbedPaneUI.this.assureRectsCreated(tabCount);
            this.calculateTabRects(BasicTabbedPaneUI.this.tabPane.getTabPlacement(), tabCount);
            BasicTabbedPaneUI.this.isRunsDirty = false;
        }
        
        private void layoutTabComponents() {
            if (BasicTabbedPaneUI.this.tabContainer == null) {
                return;
            }
            Rectangle rectangle = new Rectangle();
            Point point = new Point(-BasicTabbedPaneUI.this.tabContainer.getX(), -BasicTabbedPaneUI.this.tabContainer.getY());
            if (BasicTabbedPaneUI.access$400(BasicTabbedPaneUI.this)) {
                BasicTabbedPaneUI.access$800(BasicTabbedPaneUI.this, 0, 0, point);
            }
            for (int i = 0; i < BasicTabbedPaneUI.this.tabPane.getTabCount(); ++i) {
                Component tabComponent = BasicTabbedPaneUI.this.tabPane.getTabComponentAt(i);
                if (tabComponent != null) {
                    BasicTabbedPaneUI.this.getTabBounds(i, rectangle);
                    Dimension preferredSize = tabComponent.getPreferredSize();
                    Insets tabInsets = BasicTabbedPaneUI.this.getTabInsets(BasicTabbedPaneUI.this.tabPane.getTabPlacement(), i);
                    int n = rectangle.x + tabInsets.left + point.x;
                    int n2 = rectangle.y + tabInsets.top + point.y;
                    int n3 = rectangle.width - tabInsets.left - tabInsets.right;
                    int n4 = rectangle.height - tabInsets.top - tabInsets.bottom;
                    int n5 = n + (n3 - preferredSize.width) / 2;
                    int n6 = n2 + (n4 - preferredSize.height) / 2;
                    int tabPlacement = BasicTabbedPaneUI.this.tabPane.getTabPlacement();
                    int n7 = i == BasicTabbedPaneUI.this.tabPane.getSelectedIndex();
                    tabComponent.setBounds(n5 + BasicTabbedPaneUI.this.getTabLabelShiftX(tabPlacement, i, (boolean)n7), n6 + BasicTabbedPaneUI.this.getTabLabelShiftY(tabPlacement, i, (boolean)n7), preferredSize.width, preferredSize.height);
                }
            }
            return;
        }
        
        protected void calculateTabRects(int n, int n2) {
            FontMetrics fontMetrics = BasicTabbedPaneUI.this.getFontMetrics();
            Dimension size = BasicTabbedPaneUI.this.tabPane.getSize();
            Insets insets = BasicTabbedPaneUI.this.tabPane.getInsets();
            Insets tabAreaInsets = BasicTabbedPaneUI.this.getTabAreaInsets(n);
            int height = fontMetrics.getHeight();
            int selectedIndex = BasicTabbedPaneUI.this.tabPane.getSelectedIndex();
            int n3 = n == 2 || n == 4;
            int leftToRight = BasicGraphicsUtils.isLeftToRight(BasicTabbedPaneUI.this.tabPane);
            int n4;
            int n5;
            int n6;
            switch (n) {
                case 2: {
                    BasicTabbedPaneUI.this.maxTabWidth = BasicTabbedPaneUI.this.calculateMaxTabWidth(n);
                    n4 = insets.left + tabAreaInsets.left;
                    n5 = insets.top + tabAreaInsets.top;
                    n6 = size.height - (insets.bottom + tabAreaInsets.bottom);
                    break;
                }
                case 4: {
                    BasicTabbedPaneUI.this.maxTabWidth = BasicTabbedPaneUI.this.calculateMaxTabWidth(n);
                    n4 = size.width - insets.right - tabAreaInsets.right - BasicTabbedPaneUI.this.maxTabWidth;
                    n5 = insets.top + tabAreaInsets.top;
                    n6 = size.height - (insets.bottom + tabAreaInsets.bottom);
                    break;
                }
                case 3: {
                    BasicTabbedPaneUI.this.maxTabHeight = BasicTabbedPaneUI.this.calculateMaxTabHeight(n);
                    n4 = insets.left + tabAreaInsets.left;
                    n5 = size.height - insets.bottom - tabAreaInsets.bottom - BasicTabbedPaneUI.this.maxTabHeight;
                    n6 = size.width - (insets.right + tabAreaInsets.right);
                    break;
                }
                default: {
                    BasicTabbedPaneUI.this.maxTabHeight = BasicTabbedPaneUI.this.calculateMaxTabHeight(n);
                    n4 = insets.left + tabAreaInsets.left;
                    n5 = insets.top + tabAreaInsets.top;
                    n6 = size.width - (insets.right + tabAreaInsets.right);
                    break;
                }
            }
            int tabRunOverlay = BasicTabbedPaneUI.this.getTabRunOverlay(n);
            BasicTabbedPaneUI.this.runCount = 0;
            BasicTabbedPaneUI.this.selectedRun = -1;
            if (n2 == 0) {
                return;
            }
            for (int i = 0; i < n2; ++i) {
                Rectangle rectangle = BasicTabbedPaneUI.this.rects[i];
                if (n3 == 0) {
                    if (i > 0) {
                        rectangle.x = BasicTabbedPaneUI.this.rects[i - 1].x + BasicTabbedPaneUI.this.rects[i - 1].width;
                    }
                    else {
                        BasicTabbedPaneUI.this.tabRuns[0] = 0;
                        BasicTabbedPaneUI.this.runCount = 1;
                        BasicTabbedPaneUI.this.maxTabWidth = 0;
                        rectangle.x = n4;
                    }
                    rectangle.width = BasicTabbedPaneUI.this.calculateTabWidth(n, i, fontMetrics);
                    BasicTabbedPaneUI.this.maxTabWidth = Math.max(BasicTabbedPaneUI.this.maxTabWidth, rectangle.width);
                    if (rectangle.x != n4 && rectangle.x + rectangle.width > n6) {
                        if (BasicTabbedPaneUI.this.runCount > BasicTabbedPaneUI.this.tabRuns.length - 1) {
                            BasicTabbedPaneUI.this.expandTabRunsArray();
                        }
                        BasicTabbedPaneUI.this.tabRuns[BasicTabbedPaneUI.this.runCount] = i;
                        BasicTabbedPaneUI this$2 = BasicTabbedPaneUI.this;
                        ++this$2.runCount;
                        rectangle.x = n4;
                    }
                    rectangle.y = n5;
                    rectangle.height = BasicTabbedPaneUI.this.maxTabHeight;
                }
                else {
                    if (i > 0) {
                        rectangle.y = BasicTabbedPaneUI.this.rects[i - 1].y + BasicTabbedPaneUI.this.rects[i - 1].height;
                    }
                    else {
                        BasicTabbedPaneUI.this.tabRuns[0] = 0;
                        BasicTabbedPaneUI.this.runCount = 1;
                        BasicTabbedPaneUI.this.maxTabHeight = 0;
                        rectangle.y = n5;
                    }
                    rectangle.height = BasicTabbedPaneUI.this.calculateTabHeight(n, i, height);
                    BasicTabbedPaneUI.this.maxTabHeight = Math.max(BasicTabbedPaneUI.this.maxTabHeight, rectangle.height);
                    if (rectangle.y != n5 && rectangle.y + rectangle.height > n6) {
                        if (BasicTabbedPaneUI.this.runCount > BasicTabbedPaneUI.this.tabRuns.length - 1) {
                            BasicTabbedPaneUI.this.expandTabRunsArray();
                        }
                        BasicTabbedPaneUI.this.tabRuns[BasicTabbedPaneUI.this.runCount] = i;
                        BasicTabbedPaneUI this$0 = BasicTabbedPaneUI.this;
                        ++this$0.runCount;
                        rectangle.y = n5;
                    }
                    rectangle.x = n4;
                    rectangle.width = BasicTabbedPaneUI.this.maxTabWidth;
                }
                if (i == selectedIndex) {
                    BasicTabbedPaneUI.this.selectedRun = BasicTabbedPaneUI.this.runCount - 1;
                }
            }
            if (BasicTabbedPaneUI.this.runCount > 1) {
                this.normalizeTabRuns(n, n2, n3 ? n5 : n4, n6);
                BasicTabbedPaneUI.this.selectedRun = BasicTabbedPaneUI.this.getRunForTab(n2, selectedIndex);
                if (BasicTabbedPaneUI.this.shouldRotateTabRuns(n)) {
                    this.rotateTabRuns(n, BasicTabbedPaneUI.this.selectedRun);
                }
            }
            for (int j = BasicTabbedPaneUI.this.runCount - 1; j >= 0; --j) {
                int k = BasicTabbedPaneUI.this.tabRuns[j];
                int n7 = BasicTabbedPaneUI.this.tabRuns[(j == BasicTabbedPaneUI.this.runCount - 1) ? 0 : (j + 1)];
                int n8 = (n7 != 0) ? (n7 - 1) : (n2 - 1);
                if (n3 == 0) {
                    for (; k <= n8; ++k) {
                        Rectangle rectangle3 = BasicTabbedPaneUI.this.rects[k];
                        rectangle3.y = n5;
                        rectangle3.x += BasicTabbedPaneUI.this.getTabRunIndent(n, j);
                    }
                    if (BasicTabbedPaneUI.this.shouldPadTabRun(n, j)) {
                        this.padTabRun(n, k, n8, n6);
                    }
                    if (n == 3) {
                        n5 -= BasicTabbedPaneUI.this.maxTabHeight - tabRunOverlay;
                    }
                    else {
                        n5 += BasicTabbedPaneUI.this.maxTabHeight - tabRunOverlay;
                    }
                }
                else {
                    for (; k <= n8; ++k) {
                        Rectangle rectangle2 = BasicTabbedPaneUI.this.rects[k];
                        rectangle2.x = n4;
                        rectangle2.y += BasicTabbedPaneUI.this.getTabRunIndent(n, j);
                    }
                    if (BasicTabbedPaneUI.this.shouldPadTabRun(n, j)) {
                        this.padTabRun(n, k, n8, n6);
                    }
                    if (n == 4) {
                        n4 -= BasicTabbedPaneUI.this.maxTabWidth - tabRunOverlay;
                    }
                    else {
                        n4 += BasicTabbedPaneUI.this.maxTabWidth - tabRunOverlay;
                    }
                }
            }
            this.padSelectedTab(n, selectedIndex);
            if (leftToRight == 0 && n3 == 0) {
                int n9 = size.width - (insets.right + tabAreaInsets.right);
                for (int l = 0; l < n2; ++l) {
                    BasicTabbedPaneUI.this.rects[l].x = n9 - BasicTabbedPaneUI.this.rects[l].x - BasicTabbedPaneUI.this.rects[l].width;
                }
            }
            return;
        }
        
        protected void rotateTabRuns(int n, int n2) {
            int n3;
            for (int i = 0; i < n2; BasicTabbedPaneUI.this.tabRuns[BasicTabbedPaneUI.this.runCount - 1] = n3, ++i) {
                n3 = BasicTabbedPaneUI.this.tabRuns[0];
                for (int j = 1; j < BasicTabbedPaneUI.this.runCount; ++j) {
                    BasicTabbedPaneUI.this.tabRuns[j - 1] = BasicTabbedPaneUI.this.tabRuns[j];
                }
            }
        }
        
        protected void normalizeTabRuns(int n, int n2, int n3, int n4) {
            int n5 = n == 2 || n == 4;
            int n6 = BasicTabbedPaneUI.this.runCount - 1;
            int i = 1;
            double n7 = 1.25;
            while (i != 0) {
                int lastTabInRun = BasicTabbedPaneUI.this.lastTabInRun(n2, n6);
                int lastTabInRun2 = BasicTabbedPaneUI.this.lastTabInRun(n2, n6 - 1);
                int n8;
                int n9;
                if (n5 == 0) {
                    n8 = BasicTabbedPaneUI.this.rects[lastTabInRun].x + BasicTabbedPaneUI.this.rects[lastTabInRun].width;
                    n9 = (int)((double)BasicTabbedPaneUI.this.maxTabWidth * n7);
                }
                else {
                    n8 = BasicTabbedPaneUI.this.rects[lastTabInRun].y + BasicTabbedPaneUI.this.rects[lastTabInRun].height;
                    n9 = (int)((double)BasicTabbedPaneUI.this.maxTabHeight * n7 * 2.0);
                }
                if (n4 - n8 > n9) {
                    BasicTabbedPaneUI.this.tabRuns[n6] = lastTabInRun2;
                    if (n5 == 0) {
                        BasicTabbedPaneUI.this.rects[lastTabInRun2].x = n3;
                    }
                    else {
                        BasicTabbedPaneUI.this.rects[lastTabInRun2].y = n3;
                    }
                    for (int j = lastTabInRun2 + 1; j <= lastTabInRun; ++j) {
                        if (n5 == 0) {
                            BasicTabbedPaneUI.this.rects[j].x = BasicTabbedPaneUI.this.rects[j - 1].x + BasicTabbedPaneUI.this.rects[j - 1].width;
                        }
                        else {
                            BasicTabbedPaneUI.this.rects[j].y = BasicTabbedPaneUI.this.rects[j - 1].y + BasicTabbedPaneUI.this.rects[j - 1].height;
                        }
                    }
                }
                else if (n6 == BasicTabbedPaneUI.this.runCount - 1) {
                    i = 0;
                }
                if (n6 - 1 > 0) {
                    --n6;
                }
                else {
                    n6 = BasicTabbedPaneUI.this.runCount - 1;
                    n7 += 0.25;
                }
            }
        }
        
        protected void padTabRun(int n, int i, int n2, int n3) {
            Rectangle rectangle = BasicTabbedPaneUI.this.rects[n2];
            if (n == 1 || n == 3) {
                float n5;
                Rectangle rectangle3;
                for (n5 = (float)(n3 - (rectangle.x + rectangle.width)) / (float)(rectangle.x + rectangle.width - BasicTabbedPaneUI.this.rects[i].x), ; i <= n2; rectangle3.width += Math.round((float)rectangle3.width * n5), ++i) {
                    rectangle3 = BasicTabbedPaneUI.this.rects[i];
                    if (i > i) {
                        rectangle3.x = BasicTabbedPaneUI.this.rects[i - 1].x + BasicTabbedPaneUI.this.rects[i - 1].width;
                    }
                }
                rectangle.width = n3 - rectangle.x;
            }
            else {
                float n4;
                Rectangle rectangle2;
                for (n4 = (float)(n3 - (rectangle.y + rectangle.height)) / (float)(rectangle.y + rectangle.height - BasicTabbedPaneUI.this.rects[i].y), ; i <= n2; rectangle2.height += Math.round((float)rectangle2.height * n4), ++i) {
                    rectangle2 = BasicTabbedPaneUI.this.rects[i];
                    if (i > i) {
                        rectangle2.y = BasicTabbedPaneUI.this.rects[i - 1].y + BasicTabbedPaneUI.this.rects[i - 1].height;
                    }
                }
                rectangle.height = n3 - rectangle.y;
            }
        }
        
        protected void padSelectedTab(int n, int n2) {
            if (n2 >= 0) {
                Rectangle rectangle = BasicTabbedPaneUI.this.rects[n2];
                Insets selectedTabPadInsets = BasicTabbedPaneUI.this.getSelectedTabPadInsets(n);
                rectangle.x -= selectedTabPadInsets.left;
                rectangle.width += selectedTabPadInsets.left + selectedTabPadInsets.right;
                rectangle.y -= selectedTabPadInsets.top;
                rectangle.height += selectedTabPadInsets.top + selectedTabPadInsets.bottom;
                if (!BasicTabbedPaneUI.access$400(BasicTabbedPaneUI.this)) {
                    Dimension size = BasicTabbedPaneUI.this.tabPane.getSize();
                    Insets insets = BasicTabbedPaneUI.this.tabPane.getInsets();
                    if (n == 2 || n == 4) {
                        int n5 = insets.top - rectangle.y;
                        if (n5 > 0) {
                            rectangle.y += n5;
                            rectangle.height -= n5;
                        }
                        int n6 = rectangle.y + rectangle.height + insets.bottom - size.height;
                        if (n6 > 0) {
                            rectangle.height -= n6;
                        }
                    }
                    else {
                        int n3 = insets.left - rectangle.x;
                        if (n3 > 0) {
                            rectangle.x += n3;
                            rectangle.width -= n3;
                        }
                        int n4 = rectangle.x + rectangle.width + insets.right - size.width;
                        if (n4 > 0) {
                            rectangle.width -= n4;
                        }
                    }
                }
            }
        }
        
        static void access$900(BasicTabbedPaneUI.TabbedPaneLayout tabbedPaneLayout) {
            tabbedPaneLayout.layoutTabComponents();
        }
    }
    
    private class TabbedPaneScrollLayout extends BasicTabbedPaneUI.TabbedPaneLayout
    {
        @Override
        protected int preferredTabAreaHeight(int n, int n2) {
            return BasicTabbedPaneUI.this.calculateMaxTabHeight(n);
        }
        
        @Override
        protected int preferredTabAreaWidth(int n, int n2) {
            return BasicTabbedPaneUI.this.calculateMaxTabWidth(n);
        }
        
        @Override
        public void layoutContainer(Container container) {
            BasicTabbedPaneUI.this.setRolloverTab(-1);
            int tabPlacement = BasicTabbedPaneUI.this.tabPane.getTabPlacement();
            int tabCount = BasicTabbedPaneUI.this.tabPane.getTabCount();
            Insets insets = BasicTabbedPaneUI.this.tabPane.getInsets();
            int selectedIndex = BasicTabbedPaneUI.this.tabPane.getSelectedIndex();
            Component visibleComponent = BasicTabbedPaneUI.this.getVisibleComponent();
            this.calculateLayoutInfo();
            Component component = null;
            if (selectedIndex < 0) {
                if (visibleComponent != null) {
                    BasicTabbedPaneUI.this.setVisibleComponent(null);
                }
            }
            else {
                component = BasicTabbedPaneUI.this.tabPane.getComponentAt(selectedIndex);
            }
            if (BasicTabbedPaneUI.this.tabPane.getTabCount() == 0) {
                BasicTabbedPaneUI.this.tabScroller.croppedEdge.resetParams();
                BasicTabbedPaneUI.this.tabScroller.scrollForwardButton.setVisible(false);
                BasicTabbedPaneUI.this.tabScroller.scrollBackwardButton.setVisible(false);
                return;
            }
            boolean n = false;
            if (component != null) {
                if (component != visibleComponent && visibleComponent != null && SwingUtilities.findFocusOwner(visibleComponent) != null) {
                    n = true;
                }
                BasicTabbedPaneUI.this.setVisibleComponent(component);
            }
            Insets contentBorderInsets = BasicTabbedPaneUI.this.getContentBorderInsets(tabPlacement);
            Rectangle bounds = BasicTabbedPaneUI.this.tabPane.getBounds();
            int componentCount = BasicTabbedPaneUI.this.tabPane.getComponentCount();
            if (componentCount > 0) {
                int width;
                int height;
                int x;
                int y;
                int x2;
                int y2;
                int width2;
                int height2;
                switch (tabPlacement) {
                    case 2: {
                        width = BasicTabbedPaneUI.this.calculateTabAreaWidth(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabWidth);
                        height = bounds.height - insets.top - insets.bottom;
                        x = insets.left;
                        y = insets.top;
                        x2 = x + width + contentBorderInsets.left;
                        y2 = y + contentBorderInsets.top;
                        width2 = bounds.width - insets.left - insets.right - width - contentBorderInsets.left - contentBorderInsets.right;
                        height2 = bounds.height - insets.top - insets.bottom - contentBorderInsets.top - contentBorderInsets.bottom;
                        break;
                    }
                    case 4: {
                        width = BasicTabbedPaneUI.this.calculateTabAreaWidth(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabWidth);
                        height = bounds.height - insets.top - insets.bottom;
                        x = bounds.width - insets.right - width;
                        y = insets.top;
                        x2 = insets.left + contentBorderInsets.left;
                        y2 = insets.top + contentBorderInsets.top;
                        width2 = bounds.width - insets.left - insets.right - width - contentBorderInsets.left - contentBorderInsets.right;
                        height2 = bounds.height - insets.top - insets.bottom - contentBorderInsets.top - contentBorderInsets.bottom;
                        break;
                    }
                    case 3: {
                        width = bounds.width - insets.left - insets.right;
                        height = BasicTabbedPaneUI.this.calculateTabAreaHeight(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabHeight);
                        x = insets.left;
                        y = bounds.height - insets.bottom - height;
                        x2 = insets.left + contentBorderInsets.left;
                        y2 = insets.top + contentBorderInsets.top;
                        width2 = bounds.width - insets.left - insets.right - contentBorderInsets.left - contentBorderInsets.right;
                        height2 = bounds.height - insets.top - insets.bottom - height - contentBorderInsets.top - contentBorderInsets.bottom;
                        break;
                    }
                    default: {
                        width = bounds.width - insets.left - insets.right;
                        height = BasicTabbedPaneUI.this.calculateTabAreaHeight(tabPlacement, BasicTabbedPaneUI.this.runCount, BasicTabbedPaneUI.this.maxTabHeight);
                        x = insets.left;
                        y = insets.top;
                        x2 = x + contentBorderInsets.left;
                        y2 = y + height + contentBorderInsets.top;
                        width2 = bounds.width - insets.left - insets.right - contentBorderInsets.left - contentBorderInsets.right;
                        height2 = bounds.height - insets.top - insets.bottom - height - contentBorderInsets.top - contentBorderInsets.bottom;
                        break;
                    }
                }
                for (int i = 0; i < componentCount; ++i) {
                    Component component2 = BasicTabbedPaneUI.this.tabPane.getComponent(i);
                    if (BasicTabbedPaneUI.this.tabScroller != null && component2 == BasicTabbedPaneUI.this.tabScroller.viewport) {
                        Rectangle viewRect = ((JViewport)component2).getViewRect();
                        Dimension preferredSize2 = BasicTabbedPaneUI.this.tabScroller.scrollForwardButton.getPreferredSize();
                        switch (tabPlacement) {
                            case 2:
                            case 4: {
                                int n2 = BasicTabbedPaneUI.this.rects[tabCount - 1].y + BasicTabbedPaneUI.this.rects[tabCount - 1].height;
                                if (n2 > height) {
                                    height = ((height > 2 * preferredSize2.height) ? (height - 2 * preferredSize2.height) : 0);
                                    if (n2 - viewRect.y <= height) {
                                        height = n2 - viewRect.y;
                                        break;
                                    }
                                    break;
                                }
                                else {
                                    break;
                                }
                            }
                            default: {
                                int n3 = BasicTabbedPaneUI.this.rects[tabCount - 1].x + BasicTabbedPaneUI.this.rects[tabCount - 1].width;
                                if (n3 > width) {
                                    width = ((width > 2 * preferredSize2.width) ? (width - 2 * preferredSize2.width) : 0);
                                    if (n3 - viewRect.x <= width) {
                                        width = n3 - viewRect.x;
                                        break;
                                    }
                                    break;
                                }
                                else {
                                    break;
                                }
                            }
                        }
                        component2.setBounds(x, y, width, height);
                    }
                    else if (BasicTabbedPaneUI.this.tabScroller != null && (component2 == BasicTabbedPaneUI.this.tabScroller.scrollForwardButton || component2 == BasicTabbedPaneUI.this.tabScroller.scrollBackwardButton)) {
                        Dimension preferredSize = component2.getPreferredSize();
                        int x3 = 0;
                        int y3 = 0;
                        int width3 = preferredSize.width;
                        int height3 = preferredSize.height;
                        boolean visible = false;
                        switch (tabPlacement) {
                            case 2:
                            case 4: {
                                if (BasicTabbedPaneUI.this.rects[tabCount - 1].y + BasicTabbedPaneUI.this.rects[tabCount - 1].height > height) {
                                    visible = true;
                                    x3 = ((tabPlacement == 2) ? (x + width - preferredSize.width) : x);
                                    y3 = ((component2 == BasicTabbedPaneUI.this.tabScroller.scrollForwardButton) ? (bounds.height - insets.bottom - preferredSize.height) : (bounds.height - insets.bottom - 2 * preferredSize.height));
                                    break;
                                }
                                break;
                            }
                            default: {
                                if (BasicTabbedPaneUI.this.rects[tabCount - 1].x + BasicTabbedPaneUI.this.rects[tabCount - 1].width > width) {
                                    visible = true;
                                    x3 = ((component2 == BasicTabbedPaneUI.this.tabScroller.scrollForwardButton) ? (bounds.width - insets.left - preferredSize.width) : (bounds.width - insets.left - 2 * preferredSize.width));
                                    y3 = ((tabPlacement == 1) ? (y + height - preferredSize.height) : y);
                                    break;
                                }
                                break;
                            }
                        }
                        component2.setVisible((boolean)visible);
                        if (!(visible == 0)) {
                            component2.setBounds(x3, y3, width3, height3);
                        }
                    }
                    else {
                        component2.setBounds(x2, y2, width2, height2);
                    }
                }
                BasicTabbedPaneUI.TabbedPaneLayout.access$900(this);
                this.layoutCroppedEdge();
                if (n && !BasicTabbedPaneUI.this.requestFocusForVisibleComponent()) {
                    BasicTabbedPaneUI.this.tabPane.requestFocus();
                }
            }
            return;
        }
        
        private void layoutCroppedEdge() {
            BasicTabbedPaneUI.this.tabScroller.croppedEdge.resetParams();
            Rectangle viewRect = BasicTabbedPaneUI.this.tabScroller.viewport.getViewRect();
            for (int i = 0; i < BasicTabbedPaneUI.this.rects.length; ++i) {
                Rectangle rectangle = BasicTabbedPaneUI.this.rects[i];
                switch (BasicTabbedPaneUI.this.tabPane.getTabPlacement()) {
                    case 2:
                    case 4: {
                        int n = viewRect.y + viewRect.height;
                        if (rectangle.y < n && rectangle.y + rectangle.height > n) {
                            BasicTabbedPaneUI.this.tabScroller.croppedEdge.setParams(i, n - rectangle.y - 1, -BasicTabbedPaneUI.this.currentTabAreaInsets.left, 0);
                            break;
                        }
                        break;
                    }
                    default: {
                        int n2 = viewRect.x + viewRect.width;
                        if (rectangle.x < n2 - 1 && rectangle.x + rectangle.width > n2) {
                            BasicTabbedPaneUI.this.tabScroller.croppedEdge.setParams(i, n2 - rectangle.x - 1, 0, -BasicTabbedPaneUI.this.currentTabAreaInsets.top);
                            break;
                        }
                        break;
                    }
                }
            }
        }
        
        @Override
        protected void calculateTabRects(int n, int n2) {
            FontMetrics fontMetrics = BasicTabbedPaneUI.this.getFontMetrics();
            Dimension size = BasicTabbedPaneUI.this.tabPane.getSize();
            Insets insets = BasicTabbedPaneUI.this.tabPane.getInsets();
            Insets tabAreaInsets = BasicTabbedPaneUI.this.getTabAreaInsets(n);
            int height = fontMetrics.getHeight();
            int selectedIndex = BasicTabbedPaneUI.this.tabPane.getSelectedIndex();
            int n3 = n == 2 || n == 4;
            int leftToRight = BasicGraphicsUtils.isLeftToRight(BasicTabbedPaneUI.this.tabPane);
            int left = tabAreaInsets.left;
            int top = tabAreaInsets.top;
            int maxTabWidth = 0;
            int height2 = 0;
            switch (n) {
                case 2:
                case 4: {
                    BasicTabbedPaneUI.this.maxTabWidth = BasicTabbedPaneUI.this.calculateMaxTabWidth(n);
                    break;
                }
                default: {
                    BasicTabbedPaneUI.this.maxTabHeight = BasicTabbedPaneUI.this.calculateMaxTabHeight(n);
                    break;
                }
            }
            BasicTabbedPaneUI.this.runCount = 0;
            BasicTabbedPaneUI.this.selectedRun = -1;
            if (n2 == 0) {
                return;
            }
            BasicTabbedPaneUI.this.selectedRun = 0;
            BasicTabbedPaneUI.this.runCount = 1;
            for (int i = 0; i < n2; ++i) {
                Rectangle rectangle = BasicTabbedPaneUI.this.rects[i];
                if (n3 == 0) {
                    if (i > 0) {
                        rectangle.x = BasicTabbedPaneUI.this.rects[i - 1].x + BasicTabbedPaneUI.this.rects[i - 1].width;
                    }
                    else {
                        BasicTabbedPaneUI.this.tabRuns[0] = 0;
                        BasicTabbedPaneUI.this.maxTabWidth = 0;
                        height2 += BasicTabbedPaneUI.this.maxTabHeight;
                        rectangle.x = left;
                    }
                    rectangle.width = BasicTabbedPaneUI.this.calculateTabWidth(n, i, fontMetrics);
                    maxTabWidth = rectangle.x + rectangle.width;
                    BasicTabbedPaneUI.this.maxTabWidth = Math.max(BasicTabbedPaneUI.this.maxTabWidth, rectangle.width);
                    rectangle.y = top;
                    rectangle.height = BasicTabbedPaneUI.this.maxTabHeight;
                }
                else {
                    if (i > 0) {
                        rectangle.y = BasicTabbedPaneUI.this.rects[i - 1].y + BasicTabbedPaneUI.this.rects[i - 1].height;
                    }
                    else {
                        BasicTabbedPaneUI.this.tabRuns[0] = 0;
                        BasicTabbedPaneUI.this.maxTabHeight = 0;
                        maxTabWidth = BasicTabbedPaneUI.this.maxTabWidth;
                        rectangle.y = top;
                    }
                    rectangle.height = BasicTabbedPaneUI.this.calculateTabHeight(n, i, height);
                    height2 = rectangle.y + rectangle.height;
                    BasicTabbedPaneUI.this.maxTabHeight = Math.max(BasicTabbedPaneUI.this.maxTabHeight, rectangle.height);
                    rectangle.x = left;
                    rectangle.width = BasicTabbedPaneUI.this.maxTabWidth;
                }
            }
            if (BasicTabbedPaneUI.this.tabsOverlapBorder) {
                this.padSelectedTab(n, selectedIndex);
            }
            if (leftToRight == 0 && n3 == 0) {
                int n4 = size.width - (insets.right + tabAreaInsets.right);
                for (int j = 0; j < n2; ++j) {
                    BasicTabbedPaneUI.this.rects[j].x = n4 - BasicTabbedPaneUI.this.rects[j].x - BasicTabbedPaneUI.this.rects[j].width;
                }
            }
            BasicTabbedPaneUI.this.tabScroller.tabPanel.setPreferredSize(new Dimension(maxTabWidth, height2));
            BasicTabbedPaneUI.this.tabScroller.tabPanel.invalidate();
            return;
        }
    }
    
    private class ScrollableTabSupport implements ActionListener, ChangeListener
    {
        public BasicTabbedPaneUI.ScrollableTabViewport viewport;
        public BasicTabbedPaneUI.ScrollableTabPanel tabPanel;
        public JButton scrollForwardButton;
        public JButton scrollBackwardButton;
        public BasicTabbedPaneUI.CroppedEdge croppedEdge;
        public int leadingTabIndex;
        private Point tabViewPosition;
        
        ScrollableTabSupport(int n) {
            this.tabViewPosition = new Point(0, 0);
            this.viewport = new BasicTabbedPaneUI.ScrollableTabViewport();
            this.tabPanel = new BasicTabbedPaneUI.ScrollableTabPanel();
            this.viewport.setView(this.tabPanel);
            this.viewport.addChangeListener(this);
            this.croppedEdge = new BasicTabbedPaneUI.CroppedEdge();
            this.createButtons();
        }
        
        void createButtons() {
            if (this.scrollForwardButton != null) {
                BasicTabbedPaneUI.this.tabPane.remove(this.scrollForwardButton);
                this.scrollForwardButton.removeActionListener(this);
                BasicTabbedPaneUI.this.tabPane.remove(this.scrollBackwardButton);
                this.scrollBackwardButton.removeActionListener(this);
            }
            int tabPlacement = BasicTabbedPaneUI.this.tabPane.getTabPlacement();
            if (tabPlacement == 1 || tabPlacement == 3) {
                this.scrollForwardButton = BasicTabbedPaneUI.this.createScrollButton(3);
                this.scrollBackwardButton = BasicTabbedPaneUI.this.createScrollButton(7);
            }
            else {
                this.scrollForwardButton = BasicTabbedPaneUI.this.createScrollButton(5);
                this.scrollBackwardButton = BasicTabbedPaneUI.this.createScrollButton(1);
            }
            this.scrollForwardButton.addActionListener(this);
            this.scrollBackwardButton.addActionListener(this);
            BasicTabbedPaneUI.this.tabPane.add(this.scrollForwardButton);
            BasicTabbedPaneUI.this.tabPane.add(this.scrollBackwardButton);
        }
        
        public void scrollForward(int n) {
            Dimension viewSize = this.viewport.getViewSize();
            Rectangle viewRect = this.viewport.getViewRect();
            if (n == 1 || n == 3) {
                if (viewRect.width >= viewSize.width - viewRect.x) {
                    return;
                }
            }
            else if (viewRect.height >= viewSize.height - viewRect.y) {
                return;
            }
            this.setLeadingTabIndex(n, this.leadingTabIndex + 1);
        }
        
        public void scrollBackward(int n) {
            if (this.leadingTabIndex == 0) {
                return;
            }
            this.setLeadingTabIndex(n, this.leadingTabIndex - 1);
            return;
        }
        
        public void setLeadingTabIndex(int n, int leadingTabIndex) {
            this.leadingTabIndex = leadingTabIndex;
            Dimension viewSize = this.viewport.getViewSize();
            Rectangle viewRect = this.viewport.getViewRect();
            switch (n) {
                case 1:
                case 3: {
                    this.tabViewPosition.x = ((this.leadingTabIndex == 0) ? 0 : BasicTabbedPaneUI.this.rects[this.leadingTabIndex].x);
                    if (viewSize.width - this.tabViewPosition.x < viewRect.width) {
                        this.viewport.setExtentSize(new Dimension(viewSize.width - this.tabViewPosition.x, viewRect.height));
                        break;
                    }
                    break;
                }
                case 2:
                case 4: {
                    this.tabViewPosition.y = ((this.leadingTabIndex == 0) ? 0 : BasicTabbedPaneUI.this.rects[this.leadingTabIndex].y);
                    if (viewSize.height - this.tabViewPosition.y < viewRect.height) {
                        this.viewport.setExtentSize(new Dimension(viewRect.width, viewSize.height - this.tabViewPosition.y));
                        break;
                    }
                    break;
                }
            }
            this.viewport.setViewPosition(this.tabViewPosition);
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            this.updateView();
        }
        
        private void updateView() {
            int tabPlacement = BasicTabbedPaneUI.this.tabPane.getTabPlacement();
            int tabCount = BasicTabbedPaneUI.this.tabPane.getTabCount();
            BasicTabbedPaneUI.this.assureRectsCreated(tabCount);
            Rectangle bounds = this.viewport.getBounds();
            Dimension viewSize = this.viewport.getViewSize();
            Rectangle viewRect = this.viewport.getViewRect();
            this.leadingTabIndex = BasicTabbedPaneUI.access$1200(BasicTabbedPaneUI.this, viewRect.x, viewRect.y);
            if (this.leadingTabIndex + 1 < tabCount) {
                switch (tabPlacement) {
                    case 1:
                    case 3: {
                        if (BasicTabbedPaneUI.this.rects[this.leadingTabIndex].x < viewRect.x) {
                            ++this.leadingTabIndex;
                            break;
                        }
                        break;
                    }
                    case 2:
                    case 4: {
                        if (BasicTabbedPaneUI.this.rects[this.leadingTabIndex].y < viewRect.y) {
                            ++this.leadingTabIndex;
                            break;
                        }
                        break;
                    }
                }
            }
            Insets contentBorderInsets = BasicTabbedPaneUI.this.getContentBorderInsets(tabPlacement);
            switch (tabPlacement) {
                case 2: {
                    BasicTabbedPaneUI.this.tabPane.repaint(bounds.x + bounds.width, bounds.y, contentBorderInsets.left, bounds.height);
                    this.scrollBackwardButton.setEnabled(viewRect.y > 0 && this.leadingTabIndex > 0);
                    this.scrollForwardButton.setEnabled(this.leadingTabIndex < tabCount - 1 && viewSize.height - viewRect.y > viewRect.height);
                    break;
                }
                case 4: {
                    BasicTabbedPaneUI.this.tabPane.repaint(bounds.x - contentBorderInsets.right, bounds.y, contentBorderInsets.right, bounds.height);
                    this.scrollBackwardButton.setEnabled(viewRect.y > 0 && this.leadingTabIndex > 0);
                    this.scrollForwardButton.setEnabled(this.leadingTabIndex < tabCount - 1 && viewSize.height - viewRect.y > viewRect.height);
                    break;
                }
                case 3: {
                    BasicTabbedPaneUI.this.tabPane.repaint(bounds.x, bounds.y - contentBorderInsets.bottom, bounds.width, contentBorderInsets.bottom);
                    this.scrollBackwardButton.setEnabled(viewRect.x > 0 && this.leadingTabIndex > 0);
                    this.scrollForwardButton.setEnabled(this.leadingTabIndex < tabCount - 1 && viewSize.width - viewRect.x > viewRect.width);
                    break;
                }
                default: {
                    BasicTabbedPaneUI.this.tabPane.repaint(bounds.x, bounds.y + bounds.height, bounds.width, contentBorderInsets.top);
                    this.scrollBackwardButton.setEnabled(viewRect.x > 0 && this.leadingTabIndex > 0);
                    this.scrollForwardButton.setEnabled(this.leadingTabIndex < tabCount - 1 && viewSize.width - viewRect.x > viewRect.width);
                    break;
                }
            }
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            ActionMap actionMap = BasicTabbedPaneUI.this.tabPane.getActionMap();
            if (actionMap != null) {
                String key;
                if (actionEvent.getSource() == this.scrollForwardButton) {
                    key = "scrollTabsForwardAction";
                }
                else {
                    key = "scrollTabsBackwardAction";
                }
                Action value = actionMap.get(key);
                if (value != null && value.isEnabled()) {
                    value.actionPerformed(new ActionEvent(BasicTabbedPaneUI.this.tabPane, 1001, null, actionEvent.getWhen(), actionEvent.getModifiers()));
                }
            }
        }
        
        @Override
        public String toString() {
            return "viewport.viewSize=" + this.viewport.getViewSize() + "\nviewport.viewRectangle=" + this.viewport.getViewRect() + "\nleadingTabIndex=" + this.leadingTabIndex + "\ntabViewPosition=" + this.tabViewPosition;
        }
        
        static void access$2600(BasicTabbedPaneUI.ScrollableTabSupport scrollableTabSupport) {
            scrollableTabSupport.updateView();
        }
    }
    
    private class ScrollableTabViewport extends JViewport implements UIResource
    {
        public ScrollableTabViewport() {
            this.setName("TabbedPane.scrollableViewport");
            this.setScrollMode(0);
            this.setOpaque(BasicTabbedPaneUI.this.tabPane.isOpaque());
            Color background = UIManager.getColor("TabbedPane.tabAreaBackground");
            if (background == null) {
                background = BasicTabbedPaneUI.this.tabPane.getBackground();
            }
            this.setBackground(background);
        }
    }
    
    private class ScrollableTabPanel extends JPanel implements UIResource
    {
        public ScrollableTabPanel() {
            super(null);
            this.setOpaque(BasicTabbedPaneUI.this.tabPane.isOpaque());
            Color background = UIManager.getColor("TabbedPane.tabAreaBackground");
            if (background == null) {
                background = BasicTabbedPaneUI.this.tabPane.getBackground();
            }
            this.setBackground(background);
        }
        
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            BasicTabbedPaneUI.this.paintTabArea(g, BasicTabbedPaneUI.this.tabPane.getTabPlacement(), BasicTabbedPaneUI.this.tabPane.getSelectedIndex());
            if (BasicTabbedPaneUI.this.tabScroller.croppedEdge.isParamsSet() && BasicTabbedPaneUI.this.tabContainer == null) {
                Rectangle rectangle = BasicTabbedPaneUI.this.rects[BasicTabbedPaneUI.this.tabScroller.croppedEdge.getTabIndex()];
                g.translate(rectangle.x, rectangle.y);
                BasicTabbedPaneUI.this.tabScroller.croppedEdge.paintComponent(g);
                g.translate(-rectangle.x, -rectangle.y);
            }
        }
        
        @Override
        public void doLayout() {
            if (this.getComponentCount() > 0) {
                this.getComponent(0).setBounds(0, 0, this.getWidth(), this.getHeight());
            }
        }
    }
    
    private class ScrollableTabButton extends BasicArrowButton implements UIResource, SwingConstants
    {
        public ScrollableTabButton(int n) {
            super(n, UIManager.getColor("TabbedPane.selected"), UIManager.getColor("TabbedPane.shadow"), UIManager.getColor("TabbedPane.darkShadow"), UIManager.getColor("TabbedPane.highlight"));
        }
    }
    
    private class Handler implements ChangeListener, ContainerListener, FocusListener, MouseListener, MouseMotionListener, PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            JTabbedPane tabbedPane = (JTabbedPane)propertyChangeEvent.getSource();
            String propertyName = propertyChangeEvent.getPropertyName();
            int access$400 = BasicTabbedPaneUI.access$400(BasicTabbedPaneUI.this);
            if (propertyName == "mnemonicAt") {
                BasicTabbedPaneUI.access$1300(BasicTabbedPaneUI.this);
                tabbedPane.repaint();
            }
            else if (propertyName == "displayedMnemonicIndexAt") {
                tabbedPane.repaint();
            }
            else if (propertyName == "indexForTitle") {
                BasicTabbedPaneUI.this.calculatedBaseline = false;
                Integer n = (Integer)propertyChangeEvent.getNewValue();
                if (BasicTabbedPaneUI.this.htmlViews != null) {
                    BasicTabbedPaneUI.this.htmlViews.removeElementAt((int)n);
                }
                this.updateHtmlViews((int)n);
            }
            else if (propertyName == "tabLayoutPolicy") {
                BasicTabbedPaneUI.this.uninstallUI(tabbedPane);
                BasicTabbedPaneUI.this.installUI(tabbedPane);
                BasicTabbedPaneUI.this.calculatedBaseline = false;
            }
            else if (propertyName == "tabPlacement") {
                if (BasicTabbedPaneUI.access$400(BasicTabbedPaneUI.this)) {
                    BasicTabbedPaneUI.this.tabScroller.createButtons();
                }
                BasicTabbedPaneUI.this.calculatedBaseline = false;
            }
            else if (propertyName == "opaque" && access$400) {
                int booleanValue = (boolean)propertyChangeEvent.getNewValue();
                BasicTabbedPaneUI.this.tabScroller.tabPanel.setOpaque((boolean)booleanValue);
                BasicTabbedPaneUI.this.tabScroller.viewport.setOpaque((boolean)booleanValue);
            }
            else if (propertyName == "background" && access$400) {
                Color color = (Color)propertyChangeEvent.getNewValue();
                BasicTabbedPaneUI.this.tabScroller.tabPanel.setBackground(color);
                BasicTabbedPaneUI.this.tabScroller.viewport.setBackground(color);
                Color color2 = (BasicTabbedPaneUI.this.selectedColor == null) ? color : BasicTabbedPaneUI.this.selectedColor;
                BasicTabbedPaneUI.this.tabScroller.scrollForwardButton.setBackground(color2);
                BasicTabbedPaneUI.this.tabScroller.scrollBackwardButton.setBackground(color2);
            }
            else if (propertyName == "indexForTabComponent") {
                if (BasicTabbedPaneUI.this.tabContainer != null) {
                    BasicTabbedPaneUI.TabContainer.access$1700(BasicTabbedPaneUI.this.tabContainer);
                }
                Component tabComponent = BasicTabbedPaneUI.this.tabPane.getTabComponentAt((int)propertyChangeEvent.getNewValue());
                if (tabComponent != null) {
                    if (BasicTabbedPaneUI.this.tabContainer == null) {
                        BasicTabbedPaneUI.access$1800(BasicTabbedPaneUI.this);
                    }
                    else {
                        BasicTabbedPaneUI.this.tabContainer.add(tabComponent);
                    }
                }
                BasicTabbedPaneUI.this.tabPane.revalidate();
                BasicTabbedPaneUI.this.tabPane.repaint();
                BasicTabbedPaneUI.this.calculatedBaseline = false;
            }
            else if (propertyName == "indexForNullComponent") {
                BasicTabbedPaneUI.this.isRunsDirty = true;
                this.updateHtmlViews((int)propertyChangeEvent.getNewValue());
            }
            else if (propertyName == "font") {
                BasicTabbedPaneUI.this.calculatedBaseline = false;
            }
        }
        
        private void updateHtmlViews(int index) {
            String title = BasicTabbedPaneUI.this.tabPane.getTitleAt(index);
            if (BasicHTML.isHTMLString(title)) {
                if (BasicTabbedPaneUI.this.htmlViews == null) {
                    BasicTabbedPaneUI.this.htmlViews = BasicTabbedPaneUI.access$1900(BasicTabbedPaneUI.this);
                }
                else {
                    BasicTabbedPaneUI.this.htmlViews.insertElementAt(BasicHTML.createHTMLView(BasicTabbedPaneUI.this.tabPane, title), index);
                }
            }
            else if (BasicTabbedPaneUI.this.htmlViews != null) {
                BasicTabbedPaneUI.this.htmlViews.insertElementAt(null, index);
            }
            BasicTabbedPaneUI.access$1300(BasicTabbedPaneUI.this);
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            JTabbedPane tabbedPane = (JTabbedPane)changeEvent.getSource();
            tabbedPane.revalidate();
            tabbedPane.repaint();
            BasicTabbedPaneUI.this.setFocusIndex(tabbedPane.getSelectedIndex(), false);
            if (BasicTabbedPaneUI.access$400(BasicTabbedPaneUI.this)) {
                BasicTabbedPaneUI.access$2000(BasicTabbedPaneUI.this);
                int selectedIndex = tabbedPane.getSelectedIndex();
                if (selectedIndex < BasicTabbedPaneUI.this.rects.length && selectedIndex != -1) {
                    BasicTabbedPaneUI.this.tabScroller.tabPanel.scrollRectToVisible((Rectangle)BasicTabbedPaneUI.this.rects[selectedIndex].clone());
                }
            }
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
            BasicTabbedPaneUI.access$2100(BasicTabbedPaneUI.this, mouseEvent.getX(), mouseEvent.getY());
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
            BasicTabbedPaneUI.this.setRolloverTab(-1);
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            if (!BasicTabbedPaneUI.this.tabPane.isEnabled()) {
                return;
            }
            int tabForCoordinate = BasicTabbedPaneUI.this.tabForCoordinate(BasicTabbedPaneUI.this.tabPane, mouseEvent.getX(), mouseEvent.getY());
            if (tabForCoordinate >= 0 && BasicTabbedPaneUI.this.tabPane.isEnabledAt(tabForCoordinate)) {
                if (tabForCoordinate != BasicTabbedPaneUI.this.tabPane.getSelectedIndex()) {
                    BasicTabbedPaneUI.this.tabPane.setSelectedIndex(tabForCoordinate);
                }
                else if (BasicTabbedPaneUI.this.tabPane.isRequestFocusEnabled()) {
                    BasicTabbedPaneUI.this.tabPane.requestFocus();
                }
            }
            return;
        }
        
        @Override
        public void mouseDragged(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
            BasicTabbedPaneUI.access$2100(BasicTabbedPaneUI.this, mouseEvent.getX(), mouseEvent.getY());
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicTabbedPaneUI.this.setFocusIndex(BasicTabbedPaneUI.this.tabPane.getSelectedIndex(), true);
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            BasicTabbedPaneUI.access$2300(BasicTabbedPaneUI.this, BasicTabbedPaneUI.this.focusIndex);
        }
        
        @Override
        public void componentAdded(ContainerEvent containerEvent) {
            JTabbedPane tabbedPane = (JTabbedPane)containerEvent.getContainer();
            Component child = containerEvent.getChild();
            if ((child instanceof UIResource)) {
                return;
            }
            BasicTabbedPaneUI.this.isRunsDirty = true;
            this.updateHtmlViews(tabbedPane.indexOfComponent(child));
            return;
        }
        
        @Override
        public void componentRemoved(ContainerEvent containerEvent) {
            JTabbedPane tabbedPane = (JTabbedPane)containerEvent.getContainer();
            if ((containerEvent.getChild() instanceof UIResource)) {
                return;
            }
            Integer n = (Integer)tabbedPane.getClientProperty("__index_to_remove__");
            if (n != null) {
                int intValue = (int)n;
                if (BasicTabbedPaneUI.this.htmlViews != null && BasicTabbedPaneUI.this.htmlViews.size() > intValue) {
                    BasicTabbedPaneUI.this.htmlViews.removeElementAt(intValue);
                }
                tabbedPane.putClientProperty("__index_to_remove__", null);
            }
            BasicTabbedPaneUI.this.isRunsDirty = true;
            BasicTabbedPaneUI.access$1300(BasicTabbedPaneUI.this);
            BasicTabbedPaneUI.access$2400(BasicTabbedPaneUI.this);
            return;
        }
    }
    
    public class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            BasicTabbedPaneUI.access$2500(BasicTabbedPaneUI.this).propertyChange(propertyChangeEvent);
        }
    }
    
    public class TabSelectionHandler implements ChangeListener
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            BasicTabbedPaneUI.access$2500(BasicTabbedPaneUI.this).stateChanged(changeEvent);
        }
    }
    
    public class MouseHandler extends MouseAdapter
    {
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            BasicTabbedPaneUI.access$2500(BasicTabbedPaneUI.this).mousePressed(mouseEvent);
        }
    }
    
    public class FocusHandler extends FocusAdapter
    {
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicTabbedPaneUI.access$2500(BasicTabbedPaneUI.this).focusGained(focusEvent);
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            BasicTabbedPaneUI.access$2500(BasicTabbedPaneUI.this).focusLost(focusEvent);
        }
    }
    
    private class TabContainer extends JPanel implements UIResource
    {
        private boolean notifyTabbedPane;
        
        public TabContainer() {
            super(null);
            this.notifyTabbedPane = true;
            this.setOpaque(false);
        }
        
        @Override
        public void remove(Component component) {
            int indexOfTabComponent = BasicTabbedPaneUI.this.tabPane.indexOfTabComponent(component);
            super.remove(component);
            if (this.notifyTabbedPane && indexOfTabComponent != -1) {
                BasicTabbedPaneUI.this.tabPane.setTabComponentAt(indexOfTabComponent, null);
            }
        }
        
        private void removeUnusedTabComponents() {
            for (Component component : this.getComponents()) {
                if (!(component instanceof UIResource) && BasicTabbedPaneUI.this.tabPane.indexOfTabComponent(component) == -1) {
                    super.remove(component);
                }
            }
        }
        
        @Override
        public boolean isOptimizedDrawingEnabled() {
            return BasicTabbedPaneUI.this.tabScroller != null && !BasicTabbedPaneUI.this.tabScroller.croppedEdge.isParamsSet();
        }
        
        @Override
        public void doLayout() {
            if (BasicTabbedPaneUI.access$400(BasicTabbedPaneUI.this)) {
                BasicTabbedPaneUI.this.tabScroller.tabPanel.repaint();
                BasicTabbedPaneUI.ScrollableTabSupport.access$2600(BasicTabbedPaneUI.this.tabScroller);
            }
            else {
                BasicTabbedPaneUI.this.tabPane.repaint(this.getBounds());
            }
        }
        
        static void access$1700(BasicTabbedPaneUI.TabContainer tabContainer) {
            tabContainer.removeUnusedTabComponents();
        }
    }
    
    private class CroppedEdge extends JPanel implements UIResource
    {
        private Shape shape;
        private int tabIndex;
        private int cropline;
        private int cropx;
        private int cropy;
        
        public CroppedEdge() {
            this.setOpaque(false);
        }
        
        public void setParams(int tabIndex, int cropline, int cropx, int cropy) {
            this.tabIndex = tabIndex;
            this.cropline = cropline;
            this.cropx = cropx;
            this.cropy = cropy;
            Rectangle bounds = BasicTabbedPaneUI.this.rects[tabIndex];
            this.setBounds(bounds);
            this.shape = BasicTabbedPaneUI.access$2700(BasicTabbedPaneUI.this.tabPane.getTabPlacement(), bounds, cropline);
            if (this.getParent() == null && BasicTabbedPaneUI.this.tabContainer != null) {
                BasicTabbedPaneUI.this.tabContainer.add(this, 0);
            }
        }
        
        public void resetParams() {
            this.shape = null;
            if (this.getParent() == BasicTabbedPaneUI.this.tabContainer && BasicTabbedPaneUI.this.tabContainer != null) {
                BasicTabbedPaneUI.this.tabContainer.remove(this);
            }
        }
        
        public boolean isParamsSet() {
            return this.shape != null;
        }
        
        public int getTabIndex() {
            return this.tabIndex;
        }
        
        public int getCropline() {
            return this.cropline;
        }
        
        public int getCroppedSideWidth() {
            return 3;
        }
        
        private Color getBgColor() {
            Container parent = BasicTabbedPaneUI.this.tabPane.getParent();
            if (parent != null) {
                Color background = parent.getBackground();
                if (background != null) {
                    return background;
                }
            }
            return UIManager.getColor("control");
        }
        
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            if (this.isParamsSet() && g instanceof Graphics2D) {
                Graphics2D graphics2D = (Graphics2D)g;
                graphics2D.clipRect(0, 0, this.getWidth(), this.getHeight());
                graphics2D.setColor(this.getBgColor());
                graphics2D.translate(this.cropx, this.cropy);
                graphics2D.fill(this.shape);
                BasicTabbedPaneUI.access$2800(BasicTabbedPaneUI.this, g);
                graphics2D.translate(-this.cropx, -this.cropy);
            }
        }
    }
}
